/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Java3D-UI
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  *
  * @(#)J3DRenderer.java	1.1 00/08/11
  *
  * Copyright (c) 2000 Sun Microsystems, Inc. All Rights Reserved.
  *
  * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
  * modify and redistribute this software in source and binary code form,
  * provided that i) this copyright notice and license appear on all copies of
  * the software; and ii) Licensee does not utilize the software in a manner
  * which is disparaging to Sun.
  *
  * This software is provided "AS IS," without a warranty of any kind. ALL
  * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
  * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
  * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
  * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
  * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
  * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
  * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
  * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
  * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGES.
  *
  * This software is not designed or intended for use in on-line control of
  * aircraft, air traffic, aircraft navigation or aircraft communications; or in
  * the design, construction, operation or maintenance of any nuclear
  * facility. Licensee represents and warrants that it will not use or
  * redistribute the Software for such purposes.
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-01-10, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * -------------------------------------------------------------------
  * TODO:
  *       - use the JavaAdvancedImagingAPI(ported by Blackdown) for the scaling if avaliable.
  */
package jaugment2.UI.java3dUI.video;


import javax.media.*;
import javax.media.renderer.VideoRenderer;
import javax.media.Format;
import javax.media.format.VideoFormat;
import javax.media.format.RGBFormat;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.Vector;

// java3d packages
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.geometry.Cylinder;

import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.behaviors.mouse.*;

  //project
import jaugment2.helpers.Logger;

/*
 * This is a JMF renderer plugin. It texure maps the incoming
 * video frame to a 3D surface, making use of Java 3D
 */
public class J3DRenderer implements VideoRenderer
{
 private static final int debug = 0;

    /*************************************************************************
     * Variables and Constants
     *************************************************************************/

    // The descriptive name of this renderer
    public static final int SS = 128;
    public static final int IW = 160;
    public static final int IH = 120;

    private static final String name = "J3DRenderer";

    protected RGBFormat inputFormat;
    protected RGBFormat supportedRGB;
    protected Format [] supportedFormats;

    protected Buffer    lastBuffer = null;

    protected int       inWidth = 0;
    protected int       inHeight = 0;
    protected Component component = null;
    protected Rectangle reqBounds = null;
    protected Rectangle bounds = new Rectangle();
    protected boolean started = false;
    protected Object lastData = null;

//    Appearance app = null;
    Background bground =  null;
//    Texture2D tex = null;
    java.awt.Component c = null;
    int count = 0;
    boolean firstFrame = true;
    int btype = 0;
    int itype = 0;
    int ttype = 0;
    boolean byRef = true;

    /*************************************************************************
     * Constructor
     *************************************************************************/

    public J3DRenderer(Background bground)//Appearance app)
    {
     this(bground, null);
    }

    public J3DRenderer(Background bground/*Appearance app*/, java.awt.Component c)
    {
//     this.app = app;
     this.bground = bground;
     this.c   = c;
        // Prepare supported input formats and preferred format

	supportedRGB =  new RGBFormat(null,
				      Format.NOT_SPECIFIED,
				      Format.byteArray,
				      Format.NOT_SPECIFIED,
				      24,
				      3, 2, 1,
				      3, Format.NOT_SPECIFIED,
				      Format.TRUE,
				      Format.NOT_SPECIFIED);

	supportedFormats = new VideoFormat[] {supportedRGB };

	String os = System.getProperty("os.name");
	System.out.println("running on " + os);
	if ( os.startsWith("W") || os.startsWith("w")) {
	    btype = BufferedImage.TYPE_3BYTE_BGR;
	    itype = ImageComponent.FORMAT_RGB;
	    ttype = Texture.RGB;
	    byRef = true;
	}else if (os.startsWith("S") || os.startsWith("s")){
	    btype = BufferedImage.TYPE_4BYTE_ABGR;
	    itype = ImageComponent.FORMAT_RGBA;
	    ttype = Texture.RGBA;
	    byRef = true;

	} else {
	    btype = BufferedImage.TYPE_3BYTE_BGR;
	    itype = ImageComponent.FORMAT_RGB;
	    ttype = Texture.RGB;
	    byRef = false;
	}

     inWidth = IW;
     inHeight = IH;

//     this.tex = createTexture();//tex;
//          tex.setCapability(Texture.ALLOW_IMAGE_WRITE);
    }

    /****************************************************************
     * Controls implementation
     ****************************************************************/

    /**
     * Returns an array of supported controls
     **/
    public Object[] getControls() {
        // No controls
        return (Object[]) new Control[0];
    }

    /**
     * Return the control based on a control type for the PlugIn.
     */
    public Object getControl(String controlType)
    {
     if(debug>0)Logger.debug.println("DEBUG: [J3DRenderer] getControl()");
       try {
          Class  cls = Class.forName(controlType);
          Object cs[] = getControls();
          for (int i = 0; i < cs.length; i++) {
             if (cls.isInstance(cs[i]))
                return cs[i];
          }
          return null;
       } catch (Exception e) {   // no such controlType or such control
         return null;
       }
    }

    /*************************************************************************
     * PlugIn implementation
     *************************************************************************/

    public String getName()
    {
     if(debug>0)Logger.debug.println("DEBUG: [J3DRenderer] getName()");
        return name;
    }

    /**
     * Opens the plugin
     */
    public void open() throws ResourceUnavailableException
    {
     if(debug>0)Logger.debug.println("DEBUG: [J3DRenderer] open()");
	firstFrame = true;

/*	scene = createSceneGraph();
	u = new SimpleUniverse(c);
	u.getViewingPlatform().setNominalViewingTransform();
	u.addBranchGraph(scene);*/

	count = 0;
	// System.out.println("end of open");

    }

    /**
     * Resets the state of the plug-in. Typically at end of media or when media
     * is repositioned.
     */
    public void reset()
    {
     if(debug>0)Logger.debug.println("DEBUG: [J3DRenderer] reset()");
        // Nothing to do
    }

    public synchronized void close()
    {
     if(debug>0)Logger.debug.println("DEBUG: [J3DRenderer] close()");
/*	View v = u.getViewer().getView();
	v.removeCanvas3D(c);
	u.removeAllLocales();
	u = null;*/

    }

    /*************************************************************************
     * Renderer implementation
     *************************************************************************/

    public void start()
    {
     if(debug>0)Logger.debug.println("DEBUG: [J3DRenderer] start()");
        started = true;
    }

    public void stop()
    {
     if(debug>0)Logger.debug.println("DEBUG: [J3DRenderer] stop()");
        started = false;
    }

    /**
     * Lists the possible input formats supported by this plug-in.
     */
    public Format [] getSupportedInputFormats()
    {
     if(debug>0)Logger.debug.println("DEBUG: [J3DRenderer] getSupportedInputFormats()");
        return supportedFormats;
    }

    /**
     * Set the data input format.
     */
    public Format setInputFormat(Format format)
    {
     if(debug>0)Logger.debug.println("DEBUG: [J3DRenderer] setInputFormat()");
        if ( format != null && format instanceof RGBFormat &&
             format.matches(supportedRGB)) {

            inputFormat = (RGBFormat) format;
            Dimension size = inputFormat.getSize();
            inWidth = size.width;
            inHeight = size.height;
     if(debug>0)Logger.debug.println("DEBUG: [J3DRenderer] in setInputFormat = " + format);
            return format;
        } else
            return null;
    }

    /**
     * Processes the data and renders it to a component
     */
    public int process(Buffer buffer)
    {
  //   Thread.currentThread().setPriority(Thread.MAX_PRIORITY);
     if(debug>3)Logger.debug.println("DEBUG: [J3DRenderer] process()");
     if( buffer.getLength() <= 0 )
       {
        if(debug>4)Logger.debug.println("DEBUG: [J3DRenderer] process() returning( buffer.getLength() <= 0 )");
        return BUFFER_PROCESSED_OK;
       }

     if( count < 0 )
       {
        if(debug>4)Logger.debug.println("DEBUG: [J3DRenderer] process() preparing return( count < 0 )");
	count ++;
	try
          {
	   Thread.currentThread().sleep(50);
	  }
        catch( Exception ex)
          {
	   ex.printStackTrace(Logger.err);
	  }

        if(debug>4)Logger.debug.println("DEBUG: [J3DRenderer] process() returning( count < 0 )");
        return BUFFER_PROCESSED_OK;
       }

     count = 0;

     byte[] rawData =(byte[])(buffer.getData());

     // System.out.println("inWidth = " + inWidth);
     // System.out.println("inHeight = " + inHeight);
     if(debug>4)Logger.debug.println("DEBUG: [J3DRenderer] process() inWidth = " + inWidth+" inHeight = " + inHeight+" SS="+SS);

     BufferedImage bimg = new BufferedImage(SS,SS, btype);
     byte[] byteData = ((DataBufferByte)bimg.getRaster().getDataBuffer()).getData();

     int op, ip, x, y;
     byte alpha_1 = (byte)0xff;
     op = 0;
     int lineStride = 3 * inWidth;

     // scale from inWidth x inHeight to SS x SS
     if( btype == BufferedImage.TYPE_3BYTE_BGR)
       {
	for(int i = 0; i < SS; i++ )
	   for( int j = 0; j < SS; j++)
              {
               x = (inWidth*j) >> 7;
               y = (inHeight*i) >> 7;

               if( x >= inWidth || y >= inHeight )
                 {
                  byteData[op++]  = 0;
                  byteData[op++]  = 0;
                  byteData[op++]  = 0;
		 }
               else
                 {
                  ip = y*lineStride + x*3;
                  byteData[op++] = rawData[ip++];
                  byteData[op++] = rawData[ip++];
                  byteData[op++] = rawData[ip++];
                 }
              }
       }
     else
       { // in 4BYTE_ABGR format
	for( int i = 0; i < SS; i++ )
           for( int j = 0; j < SS; j++)
              {
               x = (inWidth*j) >> 7;
               y = (inHeight*i) >> 7;

               if( x >= inWidth || y >= inHeight )
                  {
                   byteData[op++] = alpha_1;
                   byteData[op++]  = 0;
                   byteData[op++]  = 0;
                   byteData[op++]  = 0;
		  }
                else
                  {
                   ip = y*lineStride + x*3;
                   byteData[op++] = alpha_1;
                   byteData[op++] = rawData[ip++];
                   byteData[op++] = rawData[ip++];
                   byteData[op++] = rawData[ip++];
		  }
              }
       }


       // scale
     if(debug>4)Logger.debug.println("DEBUG: [J3DRenderer] process() c = "+ (c==null?"null":"non-null"));
     ImageComponent2D imgcmp = null;
     if(c!=null) // do not scale
       {
        int newwidth  = c.getWidth();
        int newheight = c.getHeight();
        BufferedImage bimg2 = new BufferedImage(newwidth,newheight, btype);
        bimg2.getGraphics().drawImage(bimg, 0, 0, newwidth, newheight, 0, 0, SS, SS, null);

        imgcmp = new ImageComponent2D(itype, bimg2, byRef, true);
       }
     else
        imgcmp = new ImageComponent2D(itype, bimg, byRef, true);

//     Logger.debug.println("DEBUG: [J3DRenderer] process() tex start");
//     tex.setImage(0, imgcmp);
//     app.setTexture(tex);
bground.setImage(imgcmp);
//     Logger.debug.println("DEBUG: [J3DRenderer] process() tex stop");

     if( firstFrame )
       {
        firstFrame = false;
        // u.addBranchGraph(scene);
	try
          {
           // give J3D more time to initialize
           Thread.currentThread().sleep(5500);
          }
        catch( Exception ex)
          {
           ex.printStackTrace(Logger.err);
          }
        }
      else
       {
	try
          {
	   Thread.currentThread().sleep(30);
	  }
        catch( Exception ex)
          {
           ex.printStackTrace(Logger.err);
          }
	}

     if(debug>4)Logger.debug.println("DEBUG: [J3DRenderer] process() normal-done");
     // System.out.println("in doProcess");
     return BUFFER_PROCESSED_OK;
    }

    //---------------------
/*
    private Texture2D createTexture()
    {
     Logger.debug.println("DEBUG: [J3DRenderer] createTexture()");
	int j = 0;
	byte alpha_1 = (byte)0xff;

	BufferedImage bimg = new BufferedImage(SS, SS, btype);
	byte[] byteData = ((DataBufferByte)bimg.getRaster().getDataBuffer()).getData();
	if ( btype == BufferedImage.TYPE_4BYTE_ABGR) {
	    j = 0;
	    for ( int i = 0; i < SS*SS; i++) {
		byteData[j] = alpha_1;
		byteData[j+1] = (byte)192;
		byteData[j+2] = (byte)0;
		byteData[j+3] = (byte)192;
		j += 4;
	    }
	} else {
	    j = 0;
	    for ( int i = 0; i < SS*SS; i++) {
		byteData[j] = (byte)192;
		byteData[j+1] = (byte)0;
		byteData[j+2] = (byte)192;
		j += 3;
	    }

	}
	ImageComponent2D imgcmp = new ImageComponent2D(itype, bimg, byRef, true);
	Texture2D tex1 = new Texture2D(Texture2D.BASE_LEVEL, ttype, SS, SS);
	tex1.setImage(0, imgcmp);
	return tex1;
    }


    /****************************************************************
     * VideoRenderer implementation
     ****************************************************************/

    /**
     * Returns an AWT component that it will render to. Returns null
     * if it is not rendering to an AWT component.
     */
    public java.awt.Component getComponent()
    {
     if(debug>0)Logger.debug.println("DEBUG: [J3DRenderer] getComponent()");
     return(component);
    }

    /**
     * Requests the renderer to draw into a specified AWT component.
     * Returns false if the renderer cannot draw into the specified
     * component.
     */
    public boolean setComponent(java.awt.Component comp)
    {
     if(debug>0)Logger.debug.println("DEBUG: [J3DRenderer] setComponent()");
     component = comp;
     return(true);
    }

    /**
     * Sets the region in the component where the video is to be
     * rendered to. Video is to be scaled if necessary. If <code>rect</code>
     * is null, then the video occupies the entire component.
     */
    public void setBounds(java.awt.Rectangle rect)
    {
     if(debug>0)Logger.debug.println("DEBUG: [J3DRenderer] setBounds()");
        reqBounds = rect;
    }

    /**
     * Returns the region in the component where the video will be
     * rendered to. Returns null if the entire component is being used.
     */
    public java.awt.Rectangle getBounds()
    {
     if(debug>0)Logger.debug.println("DEBUG: [J3DRenderer] getBounds()");
        return reqBounds;
    }

    /*************************************************************************
     * Local methods
     *************************************************************************/

    int getInWidth()
    {
     if(debug>0)Logger.debug.println("DEBUG: [J3DRenderer] getInWidth()");
     return(inWidth);
    }

    int getInHeight()
    {
     if(debug>0)Logger.debug.println("DEBUG: [J3DRenderer] getInHeight()");
     return(inHeight);
    }

}
