/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Java3D-UI
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-01-10, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2001-01-11, Marcus Wolschon<Suran@gmx.net>)
  *       - cleaned up
  * -------------------------------------------------------------------
  * TODO:
  *       - reimplement capture
  */

package jaugment2.UI.java3dUI.video;

  //JDK
import java.io.File;
import java.net.URL;
  //JMF
import javax.media.*;
import javax.media.Processor;
import javax.media.format.*;
import javax.media.control.*;
  //Java3D
import com.sun.j3d.utils.geometry.Sphere;
import javax.media.j3d.*;
import javax.media.j3d.Appearance;
import javax.vecmath.*;
  //project
import jaugment2.helpers.Logger;
import jaugment2.UI.java3dUI.RemoteBackground;

/**
 * Provides life-video for the background
 * set "jaugment2.UI.java3dUI.video.url" to the URL of the source
 * examples:
 *  "v4l://0" for the fist Video4Linux-camera
 *  "http://java.sun.com/products/java-media/jmf/2.1/samples/samples/media/darkcity.7.160x120.11khz.mov" for a movie-trailer
 * If "jaugment2.UI.java3dUI.video.noscale" is set to any value
 * no scaling to full-screen will be done
 */
public class VideoBackground implements ControllerListener , RemoteBackground
{
 /**
  * helper-object to syncronize while waiting for state-transitions on
  * our data-sourceprocessor
  */
 int[] waitSync = new int[0];

 /**
  * processor for our data-source
  */
 Processor p = null;

 /**
  * this class is set as the renderer of the data-stream and
  * it updates the image of the Background on every frame it gets
  */
 J3DRenderer j3dRenderer = null;

 /**
  * helper-variable
  */
 boolean stateTransOK = true;

 private static final boolean debug = false;

  public VideoBackground()
  {
  }


//  private Appearance bgappearance = null;

  public Background getBackground(java.awt.Component c)
  {
   if(debug)Logger.debug.println("DEBUG: [VideoBackground:getBackground()] entering");
   Background bground = new Background();
   bground.setCapability(Background.ALLOW_IMAGE_WRITE);
   try
     {
      BoundingSphere bgbounds = new BoundingSphere(new Point3d(), 10000.0);//new BoundingSphere(new Point3d(0.0,0.0,0.0),10000.0);
      bground.setApplicationBounds(bgbounds);
/*      BranchGroup    bggroup = new BranchGroup();
      Appearance     bgappearance = new Appearance();
                     bgappearance.setCapability(Appearance.ALLOW_TEXTURE_WRITE);


 //------------------- video-try ------------
     TextureAttributes textAttr = new TextureAttributes();
                       textAttr.setTextureMode(TextureAttributes.MODULATE);
                     bgappearance.setTextureAttributes(textAttr);*/

        // get a film from sun.com

      String url = System.getProperty("jaugment2.UI.java3dUI.video.url");
      if(debug)Logger.debug.println("url="+url);
//      if(url == null)
//         url = "v4l://0";       //capture from first Video4Linux-device

//      URL url = new URL("http://java.sun.com/products/java-media/jmf/2.1/samples/samples/media/darkcity.7.160x120.11khz.mov");
        // play a local copy
//      URL url = new URL("file:///home/fox/darkcity.7.160x120.11khz.mov");
        // capture from the first Video4Linux-device
//      String url = new String("v4l://0");
      String noscale = System.getProperty("jaugment2.UI.java3dUI.video.noscale");
      open(new MediaLocator(url),bground, (noscale==null?c:null));/*
//      this.bgappearance  = bgappearance;

 //------------------- color-try ------------

/-      bgappearance.setColoringAttributes( new ColoringAttributes(new Color3f(1.0f,0.0f,0.0f),ColoringAttributes.ALLOW_COLOR_READ));
      Material mat = new Material();
      mat.setAmbientColor(0.0f,0.0f,1.0f);
      mat.setDiffuseColor(0.0f,0.0f,1.0f);
      mat.setEmissiveColor(0.0f,0.0f,1.0f);
      bgappearance.setMaterial(mat);-/
 //------------------- --------- ------------

      Sphere         bgsphere = new Sphere(1.0f, Sphere.GENERATE_NORMALS | Sphere.GENERATE_NORMALS_INWARD | Sphere.GENERATE_TEXTURE_COORDS, 45 , bgappearance );
                     bggroup.addChild(bgsphere);
                     bground.setGeometry(bggroup);
                     bground.setApplicationBounds(bgbounds);

*/
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     }

   if(debug)Logger.debug.println("DEBUG: [VideoBackground:getBackground()] leaving");
   return(bground);
  }

  public void start()
  {
   if(debug)Logger.debug.println("DEBUG: [VideoBackground:start()] entering");
   try
     {

      Thread t = new Thread()
      {
       public void run()
       {
        try
          {
           p.start();
           double timed=0;
           timed = p.getMediaTime().getSeconds();
        if(debug)
        while(true)
          {
               Thread.currentThread().sleep(1000);
           Logger.debug.println("DEBUG: [VideoBackground:start()] state="+p.getState() +" MediaTime="+p.getMediaTime().getSeconds()+"s"+" syncTime="+p.getSyncTime().getSeconds()+"s"        );

/*           if(timed == p.getSyncTime().getSeconds())
             {
              Time tx = new Time(p.getMediaTime().getSeconds()+1);
              p.setMediaTime(tx);
//              p.syncStart(tx);
             }*/

          }
          }
        catch(Exception e)
          {
           e.printStackTrace(Logger.err);
          }
        }};
       t.setDaemon(true);
       t.start();
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     }
   if(debug)Logger.debug.println("DEBUG: [VideoBackground:start()] leaving");
  }


  /**
   * open the media given and construct the J3DRenderer
   */
  private boolean open(MediaLocator ml, Background bground, java.awt.Component c)//Appearance app)
  {
   try
     {
      p = Manager.createProcessor(ml);
     }
   catch(Exception e)
     {
      Logger.err.println("ERROR: [VideoBackground] failed to create a processor for movie \"" + ml+"\"  ["+e.getClass().toString()+"]");
      e.printStackTrace(Logger.err);
      return(false);
     }

   p.addControllerListener(this);
   p.configure();

   if(!waitForState(p.Configured))
     {
      Logger.err.println("ERROR: [VideoBackground] Failed to configure the processor");
      return(false);
     }

   // use processor as a player
   p.setContentDescriptor(null);

   // obtain the track control
   TrackControl[] tc = p.getTrackControls();


   if( tc == null )
     {
      Logger.err.println("ERROR: [VideoBackground] Failed to get the track control from processor");
      return(false);
     }

   TrackControl vtc = null;

   for( int i =0; i < tc.length; i++ )
      {
       if(tc[i].getFormat() instanceof VideoFormat )
         {
	  vtc = tc[i];
	  break;
	 }
      }

   if( vtc == null )
     {
      Logger.err.println("ERROR: [VideoBackground] can't find video track");
      return(false);
     }

   try
     {
     // this.j3dRenderer = new J3DRenderer(app, null);
      this.j3dRenderer = new J3DRenderer(bground, c);
//      j3dRenderer.getComponent().setSize(296, 240);
      vtc.setRenderer(j3dRenderer);
     }
   catch ( Exception e)
     {
      Logger.err.println("ERROR: [VideoBackground] the processor does not support effect ["+e.getClass().toString()+"]");
      e.printStackTrace(Logger.err);
      return(false);
     }

   // prefetch
   p.prefetch();

   if(!waitForState(p.Prefetched))
     {
      Logger.err.println("ERROR: [VideoBackground] Failed to prefech the processor");
      return(false);
     }




   if(debug)Logger.debug.println("DEBUG: [VideoBackground] open(...) done");
   return(true);
  }


  //------------------------- helper
  /**
   * helper-function
   */
  public boolean waitForState(int state)
  {
   synchronized (waitSync)
    {
     try
       {
	while(p.getState() != state && stateTransOK )
          {
	   waitSync.wait();
	  }
	}
     catch (Exception ex)
        {
         ex.printStackTrace(Logger.err);
        }
     return(stateTransOK);
    }
  }

  //------------------------- ControllListener

  /**
   * This function is invoked by processor uppon state-transitions.
   */
  public void controllerUpdate(ControllerEvent evt)
  {
   if( evt instanceof ConfigureCompleteEvent ||
       evt instanceof RealizeCompleteEvent ||
       evt instanceof PrefetchCompleteEvent )
     {
      if(debug)Logger.debug.println("DEBUG: [VideoBackground] controllerUpdate(*CompleteEvent["+evt.getClass().toString()+"])");
      synchronized (waitSync)
      {
       stateTransOK = true;
       waitSync.notifyAll();
      }
     }
   else
   if( evt instanceof ResourceUnavailableEvent)
     {
      Logger.err.println("ERROR: [VideoBackground] controllerUpdate(ResourceUnavailableEvent) "+evt.toString());
      synchronized (waitSync)
      {
       stateTransOK = false;
       waitSync.notifyAll();
      }
    }
   else
   if( evt instanceof ControllerErrorEvent)
     {
      ControllerErrorEvent e = (ControllerErrorEvent)evt;

      Logger.err.println("ERROR: [VideoBackground] controllerUpdate(ControllerErrorEvent) \""+e.getMessage()+"\"");
    }
   else
   if( evt instanceof EndOfMediaEvent)
     {
      if(debug)Logger.debug.println("DEBUG: [VideoBackground] controllerUpdate(EndOfMediaEvent)");
      p.setMediaTime(new Time(0));
      p.start();
      // p.close();
     }
 /*  else
   if( evt instanceof StartEvent)
     {
      Logger.debug.println("DEBUG: [VideoBackground] controllerUpdate(StartEvent)");
      synchronized (waitSync)
      {
       stateTransOK = true;
       waitSync.notifyAll();
      }
     }*/
   else
     if(debug)Logger.debug.println("DEBUG: [VideoBackground] controllerUpdate(*UnknownEvent["+evt.getClass().toString()+"])");
  }
}

