/*
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


/* Modified by Marcus Wolschon for the jAugment-project
*/
package jaugment2.UI.java3dUI.window2D;

  //JDK
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.awt.image.*;
  //Java3D
import javax.media.j3d.*;
import javax.media.j3d.Canvas3D;
import javax.vecmath.*;
import com.sun.j3d.utils.picking.*;
import com.sun.j3d.utils.image.TextureLoader;
  //project
import jaugment2.helpers.Logger;


/**
 * ExamineViewerBehavior
 *
 * @version		1.0, 98/04/16
 **/

/**
 * Wakeup on mouse button presses, releases, and mouse movements and
 * generate transforms in an "examination style" that enables the user
 * to rotate, translation, and zoom an object as if it is held at
 * arm's length.  Such an examination style is similar to the "Examine"
 * navigation type used by VRML browsers.
 *
 * The behavior maps mouse drags to different transforms depending
 * upon the mosue button held down:
 *
 *   Button 1 (left)
 *     Horizontal movement --> Y-axis rotation
 *     Vertical movement --> X-axis rotation
 *
 *   Button 2 (middle)
 *     Horizontal movement --> nothing
 *     Vertical movement --> Z-axis translation
 *
 *   Button 3 (right)
 *     Horizontal movement --> X-axis translation
 *     Vertical movement --> Y-axis translation
 *
 * To support systems with 2 or 1 mouse buttons, the following
 * alternate mappings are supported while dragging with any mouse
 * button held down and zero or more keyboard modifiers held down:
 *
 *   No modifiers = Button 1
 *   ALT = Button 2
 *   Meta = Button 3
 *   Control = Button 3
 *
 * The behavior automatically modifies a TransformGroup provided
 * to the constructor.  The TransformGroup's transform can be set
 * at any time by the application or other behaviors to cause the
 * examine rotation and translation to be reset.
 */

// This class is inspired by the MouseBehavior, MouseRotate,
// MouseTranslate, and MouseZoom utility behaviors provided with
// Java 3D.  This class differs from those utilities in that it:
//
//    (a) encapsulates all three behaviors into one in order to
//        enforce a specific "Examine" symantic
//
//    (b) supports set/get of the rotation and translation factors
//        that control the speed of movement.
//
//    (c) supports the "Control" modifier as an alternative to the
//        "Meta" modifier not present on PC, Mac, and most non-Sun
//        keyboards.  This makes button3 behavior usable on PCs,
//        Macs, and other systems with fewer than 3 mouse buttons.

public class ExamineViewerBehavior
	extends ViewerBehavior
{
	// Previous cursor location
	protected int previousX = 0;
	protected int previousY = 0;

	// Saved standard cursor
	protected Cursor savedCursor = null;


	/**
	 * Construct an examine behavior that listens to mouse movement
	 * and button presses to generate rotation and translation
	 * transforms written into a transform group given later
	 * with the setTransformGroup( ) method.
	 */
/*	public ExamineViewerBehavior( )
	{
		super( );
	}
*/

	/**
	 * Construct an examine behavior that listens to mouse movement
	 * and button presses to generate rotation and translation
	 * transforms written into a transform group given later
	 * with the setTransformGroup( ) method.
	 *
	 * @param parent The AWT Component that contains the area
	 * generating mouse events.
	 */
/*	public ExamineViewerBehavior( Component parent )
	{
		super( parent );
	}
*/

	/**
	 * Construct an examine behavior that listens to mouse movement
	 * and button presses to generate rotation and translation
	 * transforms written into the given transform group.
	 *
	 * @param transformGroup The transform group to be modified
	 * by the behavior.
	 */
/*	public ExamineViewerBehavior( TransformGroup transformGroup )
	{
		super( );
		subjectTransformGroup = transformGroup;
	}


*/	/**
	 * Construct an examine behavior that listens to mouse movement
	 * and button presses to generate rotation and translation
	 * transforms written into the given transform group.
	 *
	 * @param transformGroup The transform group to be modified
	 * by the behavior.
	 * @param parent The AWT Component that contains the area
	 * generating mouse events.
	 */
 public ExamineViewerBehavior( BranchGroup bg, TransformGroup transformGroup, Canvas3D parent, Component ctl, Appearance app )
 {
  super( parent );
  init_check(bg, parent);
  subjectTransformGroup = transformGroup;
  this.ctl = ctl;
  this.app = app;
 }
 	/**
	 * Respond to a button1 event (press, release, or drag).
	 *
	 * @param mouseEvent A MouseEvent to respond to.
	 */
	public void onButton1( MouseEvent /*mev*/me2 )
	{
/*		if ( subjectTransformGroup == null )
			return;
//		int x = mev.getX( );
//		int y = mev.getY( );

//TODO: TEST: if we hit one of the _our_ small boxen
// if(!check(x, y))
if(getClickPosition(mev) != null)
   return;

		if ( mev.getID( ) == MouseEvent.MOUSE_PRESSED )
		{
			// Mouse button pressed:  record position
			previousX = x;
			previousY = y;

			// Change to a "move" cursor
			if ( parentComponent != null )
			{
				savedCursor = parentComponent.getCursor( );
				parentComponent.setCursor(
					Cursor.getPredefinedCursor( Cursor.HAND_CURSOR ) );
			}
			return;
		}
		if ( mev.getID( ) == MouseEvent.MOUSE_RELEASED )
		{
			// Mouse button released:  do nothing

			// Switch the cursor back
			if ( parentComponent != null )
				parentComponent.setCursor( savedCursor );
			return;
		}

		//
		// Mouse moved while button down:  create a rotation
		//
		// Compute the delta in X and Y from the previous
		// position.  Use the delta to compute rotation
		// angles with the mapping:
		//
		//   positive X mouse delta --> positive Y-axis rotation
		//   positive Y mouse delta --> positive X-axis rotation
		//
		// where positive X mouse movement is to the right, and
		// positive Y mouse movement is **down** the screen.
		//
		int deltaX = x - previousX;
		int deltaY = y - previousY;

		if ( deltaX > UNUSUAL_XDELTA || deltaX < -UNUSUAL_XDELTA ||
			deltaY > UNUSUAL_YDELTA || deltaY < -UNUSUAL_YDELTA )
		{
			// Deltas are too huge to be believable.  Probably a glitch.
			// Don't record the new XY location, or do anything.
			return;
		}

		double xRotationAngle = deltaY * XRotationFactor;
		double yRotationAngle = deltaX * YRotationFactor;

		//
		// Build transforms
		//
		transform1.rotX( xRotationAngle );
		transform2.rotY( yRotationAngle );

		// Get and save the current transform matrix
		subjectTransformGroup.getTransform( currentTransform );
		currentTransform.get( matrix );
		translate.set( matrix.m03, matrix.m13, matrix.m23 );

		// Translate to the origin, rotate, then translate back
		currentTransform.setTranslation( origin );
		currentTransform.mul( transform1, currentTransform );
		currentTransform.mul( transform2, currentTransform );
		currentTransform.setTranslation( translate );

		// Update the transform group
		subjectTransformGroup.setTransform( currentTransform );

		previousX = x;
		previousY = y;*/


            int pos[] = getClickPosition( me2 );
            if(pos == null)
              return; // our object is not hit

            int _x = pos[0];
            int _y = pos[1];




	     System.out.println ("denormalized Coordinates of intersection pt: "+_x+" : "+_y);

             Component ctl2       = ctl;
             Component parent_ctl = ctl;
             boolean changed = true;

             while(changed)
               {
                ctl2 = parent_ctl.getComponentAt(_x, _y);
                if(ctl2==null)
                  {
                   Logger.err.println ("ERROR: [Window3DBehavior] no combonent at "+_x+" : "+_y);
                   ctl2 = ctl.locate(_x, _y);
                   if(ctl2==null)
                      Logger.err.println ("ERROR: [Window3DBehavior] not within combonent at "+_x+" : "+_y);
                  }
                else
                  {
                   Logger.debug.println("DEBUG: [Window3DBehavior] ctl at "+_x+" : "+_y+" is a ["+ctl2.getClass().getName()+"]");
                  }


                if(ctl2 != null)
                  {
                   focus_c = ctl2;
                   focus_b = this;
                   _x = _x- ctl2.getX();
                   _y = _y- ctl2.getY();
                   changed = (ctl2 != parent_ctl);
                   parent_ctl = ctl2;
                  }
                else
                  {
                   ctl2 = ctl;
                   changed = false;
                  }
               }


             MouseEvent me = new MouseEvent( ctl2/* me2.getComponent()*/, me2.getID(), me2.getWhen(), me2.getModifiers(),
                                            _x, _y , me2.getClickCount(), me2.isPopupTrigger());


             ctl2.dispatchEvent(me);

                // leave a mark where we clicked
/*                Graphics  g =  ctl.getGraphics();
                g.setColor(Color.black);
                g.fillRect(_x,_y,14,14);*/

               // update the texture
               //TODO: make this faster
//----

/*            if(ctl.getWidth()>0 && ctl.getHeight()>0)
              {
               ctl.setSize(100, 100);
              }*/

            updateGraphics();
/*            if(ctl.getWidth()>0 && ctl.getHeight()>0)
              {
               BufferedImage img = new BufferedImage(ctl.getWidth(), ctl.getHeight(), BufferedImage.TYPE_INT_RGB);
               Graphics ctlg = img.getGraphics();
               ctl.paintAll(ctlg);
               ctlg.drawLine(0,0, 59, 25);
               ctlg.drawLine(0, 25, 59, 0);
               Texture txt = (new TextureLoader(img)).getTexture();
               app.setTexture(txt);
              }
            else
              {
               Logger.warning.println("WARNING: [Window2DBehavior] control has size 0");
              }*/
//----
            System.err.println("WARNING: [Window2DBehavior] done");

	}


	/**
	 * Respond to a button2 event (press, release, or drag).
	 *
	 * @param mouseEvent A MouseEvent to respond to.
	 */
	public void onButton2( MouseEvent mev )
	{
		if ( subjectTransformGroup == null )
			return;

		int x = mev.getX( );
		int y = mev.getY( );

//TODO: TEST: if we hit one of the _our_ small boxen
 if(!check(x, y))
   return;

		if ( mev.getID( ) == MouseEvent.MOUSE_PRESSED )
		{
			// Mouse button pressed:  record position
			previousX = x;
			previousY = y;

			// Change to a "move" cursor
			if ( parentComponent != null )
			{
				savedCursor = parentComponent.getCursor( );
				parentComponent.setCursor(
					Cursor.getPredefinedCursor( Cursor.MOVE_CURSOR ) );
			}
			return;
		}
		if ( mev.getID( ) == MouseEvent.MOUSE_RELEASED )
		{
			// Mouse button released:  do nothing

			// Switch the cursor back
			if ( parentComponent != null )
				parentComponent.setCursor( savedCursor );
			return;
		}

		//
		// Mouse moved while button down:  create a translation
		//
		// Compute the delta in Y from the previous
		// position.  Use the delta to compute translation
		// distances with the mapping:
		//
		//   positive Y mouse delta --> positive Y-axis translation
		//
		// where positive X mouse movement is to the right, and
		// positive Y mouse movement is **down** the screen.
		//
		int deltaY = y - previousY;

		if ( deltaY > UNUSUAL_YDELTA || deltaY < -UNUSUAL_YDELTA )
		{
			// Deltas are too huge to be believable.  Probably a glitch.
			// Don't record the new XY location, or do anything.
			return;
		}

		double zTranslationDistance = deltaY * ZTranslationFactor;

		//
		// Build transforms
		//
		translate.set( 0.0, 0.0, zTranslationDistance );
		transform1.set( translate );

		// Get and save the current transform
		subjectTransformGroup.getTransform( currentTransform );

		// Translate as needed
		currentTransform.mul( transform1, currentTransform );

		// Update the transform group
		subjectTransformGroup.setTransform( currentTransform );

		previousX = x;
		previousY = y;
	}


	/**
	 * Respond to a button3 event (press, release, or drag).
	 *
	 * @param mouseEvent A MouseEvent to respond to.
	 */
	public void onButton3( MouseEvent mev )
	{
		if ( subjectTransformGroup == null )
			return;

		int x = mev.getX( );
		int y = mev.getY( );

//TODO: TEST: if we hit one of the _our_ small boxen
 if(!check(x, y))
   return;

		if ( mev.getID( ) == MouseEvent.MOUSE_PRESSED )
		{
			// Mouse button pressed:  record position
			previousX = x;
			previousY = y;

			// Change to a "move" cursor
			if ( parentComponent != null )
			{
				savedCursor = parentComponent.getCursor( );
				parentComponent.setCursor(
					Cursor.getPredefinedCursor( Cursor.MOVE_CURSOR ) );
			}
			return;
		}
		if ( mev.getID( ) == MouseEvent.MOUSE_RELEASED )
		{
			// Mouse button released:  do nothing

			// Switch the cursor back
			if ( parentComponent != null )
				parentComponent.setCursor( savedCursor );
			return;
		}

		//
		// Mouse moved while button down:  create a translation
		//
		// Compute the delta in X and Y from the previous
		// position.  Use the delta to compute translation
		// distances with the mapping:
		//
		//   positive X mouse delta --> positive X-axis translation
		//   positive Y mouse delta --> negative Y-axis translation
		//
		// where positive X mouse movement is to the right, and
		// positive Y mouse movement is **down** the screen.
		//
		int deltaX = x - previousX;
		int deltaY = y - previousY;

		if ( deltaX > UNUSUAL_XDELTA || deltaX < -UNUSUAL_XDELTA ||
			deltaY > UNUSUAL_YDELTA || deltaY < -UNUSUAL_YDELTA )
		{
			// Deltas are too huge to be believable.  Probably a glitch.
			// Don't record the new XY location, or do anything.
			return;
		}

		double xTranslationDistance = deltaX * XTranslationFactor;
		double yTranslationDistance = -deltaY * YTranslationFactor;

		//
		// Build transforms
		//
		translate.set( xTranslationDistance, yTranslationDistance, 0.0 );
		transform1.set( translate );

		// Get and save the current transform
		subjectTransformGroup.getTransform( currentTransform );

		// Translate as needed
		currentTransform.mul( transform1, currentTransform );

		// Update the transform group
		subjectTransformGroup.setTransform( currentTransform );

		previousX = x;
		previousY = y;
	}


	/**
	 * Respond to an elapsed frames event (assuming subclass has set up a
	 * wakeup criterion for it).
	 *
	 * @param time A WakeupOnElapsedFrames criterion to respond to.
	 */
	public void onElapsedFrames( WakeupOnElapsedFrames timeEvent )
	{
		// Can't happen
	}

 private PickCanvas pickCanvas = null;
  PickResult[] pickResult;
  /**
   * The component we process and translate the mouse-events for
   */
  private Component ctl;

  /**
   * The apearance to render the texture with the view of the component into
   */
  private Appearance app;

  /**
   * The component, that has the keyboard-focus
   */
  public static Component focus_c;
  /**
   * The behavior for the component, that has the keyboard-focus
   */
  public static ExamineViewerBehavior focus_b;

 private void init_check(BranchGroup branchGroup, Canvas3D canvas3D)
 {
   pickCanvas = new PickCanvas(canvas3D, branchGroup);
   pickCanvas.setTolerance(5.0f);
   pickCanvas.setMode(PickCanvas.GEOMETRY);
 }

 /**
  * Check if our geometry was picked
  */
 private boolean check(int x, int y)
 {
//  Logger.debug.println("DEBUG: [ExamineViewBehavior] check(x="+x+", y="+y+")");
  pickCanvas.setShapeLocation(x, y);
 // PickResult pr[] = pickCanvas.pickAllSorted();
   PickResult pr = pickCanvas.pickClosest();
  Logger.debug.println("DEBUG: [ExamineViewBehavior] check(x="+x+", y="+y+") returning "+(pr != null));
  return(pr != null);
//  return(true);
 }


  private int[] getClickPosition(MouseEvent evt)
  {
   System.err.println("WARNING: [Window2DBehavior] getClickPosition");
   int x = evt.getX();
   int y = evt.getY();
   pickCanvas.setShapeLocation(x, y);

   Point3d eyePos = pickCanvas.getStartPosition();
   pickResult = pickCanvas.pickAllSorted();//TODO: catch exception if we picked the small boxen and ignore the click
   if(pickResult == null)
     {
      System.err.println("WARNING: [Window2DBehavior] pickResult == null");
      return(null);
     }


   // Get closest intersection results
   PickIntersection pi = pickResult[0].getClosestIntersection(eyePos);

   GeometryArray curGeomArray = pi.getGeometryArray();

   // Position sphere at intersection point
   Vector3d v = new Vector3d();
   Point3d intPt = pi.getPointCoordinatesVW();

   Point3d []ptw = pi.getPrimitiveCoordinatesVW();
   Point3d []pt  = pi.getPrimitiveCoordinates();

   System.out.println ("=============");
   System.out.println ("Coordinates of intersection pt:"+intPt);
   System.out.println ("Coordinates of vertices: ");

   double minx = Double.MAX_VALUE;
   double miny = Double.MAX_VALUE;
   double maxx = Double.MIN_VALUE;
   double maxy = Double.MIN_VALUE;

   for(int k=0;k<pt.length;k++)
      {
       System.out.println (k + ":" + ptw[k].x + " " + ptw[k].y + " " + ptw[k].z);
       if(ptw[k].x<minx)
          minx = ptw[k].x;
       if(ptw[k].y<miny)
          miny = ptw[k].y;
       if(ptw[k].x>maxx)
          maxx = ptw[k].x;
       if(ptw[k].y>maxy)
          maxy = ptw[k].y;
      }

   double normalx = (intPt.x-minx)/(maxx-minx);
   double normaly = (intPt.y-miny)/(maxy-miny);
   System.out.println ("normalized Coordinates of intersection pt: "+normalx+" : "+normaly+" max=["+maxx+":"+maxy+"], min=["+minx+":"+miny+"]");

   if(ctl==null)
     {
      System.err.println("ERROR: [Window2DBehavior] ctl==null");
      return(null);
     }

   int _x = (int)(ctl.getWidth()*normalx);//+ctl.getX();
   int _y = (int)(ctl.getHeight()*(1-normaly));//+ctl.getY();

   Point3d closestVert = pi.getClosestVertexCoordinatesVW();
   System.out.println ("Closest vertex: "+closestVert);


   return(new int[] {_x, _y});
  }


 /**
  * Recreate and draw the texture
  */
 public void updateGraphics()
 {
  if(ctl.getWidth()>0 && ctl.getHeight()>0)
    {
     BufferedImage img = new BufferedImage(ctl.getWidth(), ctl.getHeight(), BufferedImage.TYPE_INT_RGB);
     Graphics ctlg = img.getGraphics();
     ctl.paintAll(ctlg);
//     ctlg.drawLine(0,0, 59, 25);
//     ctlg.drawLine(0, 25, 59, 0);
     Texture txt = (new TextureLoader(img)).getTexture();
     app.setTexture(txt);
    }
  else
   {
    Logger.warning.println("WARNING: [Window2DBehavior] control has size 0");
   }

 }

}
