/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Java3D-UI
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2000-12-13, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2000-12-14, Marcus Wolschon<Suran@gmx.net>)
  *       - using a bounding-box (seems to be too large)
  *       - transform3D
  *       - clicking and transforming works
  * 0.0.3 (2000-12-23, Marcus Wolschon<Suran@gmx.net>)
  *       - uses Window2DExamineViewerBehavior
  *       - no more cross-interacting with multiple windows
  *
  * -------------------------------------------------------------------
  * TODO:
  *       - provide an LOD for windows to let them apear as colored boxes
  */

package jaugment2.UI.java3dUI.window2D;


import java.awt.Component;
import java.awt.image.*;
import java.awt.Graphics;

import javax.media.j3d.*;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.OrientedShape3D;
import javax.vecmath.*;
import javax.vecmath.Point3d;
//import com.sun.j3d.utils.picking.*;
import com.sun.j3d.utils.image.TextureLoader;
import com.sun.j3d.utils.picking.PickCanvas;
import com.sun.j3d.utils.picking.PickTool;
import com.sun.j3d.utils.geometry.ColorCube;
import com.sun.j3d.loaders.Scene;
import com.sun.j3d.loaders.vrml97.VrmlLoader;
import com.sun.j3d.loaders.vrml97.VrmlScene;
import com.sun.j3d.loaders.vrml97.impl.MovieTexture;
  //project
import jaugment2.helpers.Logger;

/**
 * The representation of a java.awt.Component in 3D-Space.
 * Does not handle keyboard-events itself, use a listener on the canvas for this
 */
public class Window2D extends BranchGroup
{
  private static final boolean debugThis = true;
  private Component ctl;
  private Canvas3D canvas;
  private TransformGroup objTrans;

  /**
   * This branch-group hosts the window as a cube
   * with a Window2DBehavior
   */
  private BranchGroup windowBG = new BranchGroup();


  /**
   * Transform us
   */
  public void transform3D(Transform3D  t)
  {
   objTrans.setTransform(t);
  }

  BoundingBox    bounds = new BoundingBox(new Point3d(-0.5f, -0.5f,  -0.5f), new Point3d(0.5f, 0.5f,  0.5f));

  public Window2D(Component ctl, Canvas3D canvas)
  {
   super();

   this.ctl = ctl;
   this.canvas = canvas;

   addChild(windowBG);

   objTrans = new TransformGroup();
   objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
   objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
   windowBG.addChild(objTrans);
//---------------------------- draw the 2D-control into a texture

    Appearance app = new Appearance();
    if(!(ctl.getWidth()>0 && ctl.getHeight()>0))
      {
       ctl.doLayout();
      }

    if(ctl.getWidth()>0 && ctl.getHeight()>0)
      {
       BufferedImage img = new BufferedImage(ctl.getWidth(), ctl.getHeight(), BufferedImage.TYPE_INT_RGB);
       Graphics g = img.getGraphics();
       ctl.paintAll(g);
       Texture txt = (new TextureLoader(img)).getTexture();
       app.setTexture(txt);
      }
    else
      {
       Logger.warning.println("WARNING: [Window2D] control has size 0");
      }
    TextureAttributes txtattr = new TextureAttributes();
                      txtattr.setTextureMode(TextureAttributes.REPLACE);

    Matrix4f txtm = getTextureTransformMatrix();
    if(txtm != null)
      {
       javax.media.j3d.Transform3D txttrans = new javax.media.j3d.Transform3D();
       txttrans.set(txtm);
       txtattr.setTextureTransform(txttrans);
      }


    app.setTextureAttributes(txtattr);
    app.setCapability(Appearance.ALLOW_TEXTURE_WRITE);

//---------------------------- create the geometry and put it all together
  String geometryFile = System.getProperty("jaugment2.UI.java3dUI.window2D.Window2D.geometryfile");

  if(geometryFile != null)
    {
     try  // load the specified geometry
       {
        VrmlLoader loader = new VrmlLoader();
        Scene s = loader.load(geometryFile);
        if(s.getNamedObjects().containsKey("Screen"))
          {
           Object o = s.getNamedObjects().get("Screen");
           setMaterial(o, app);
          }
        objTrans.addChild(s.getSceneGroup());
       }
    catch(Exception e)
      {
       e.printStackTrace(Logger.err);
       geometryFile = null;
      }
   }

    //default-geometry (cube)
  if(geometryFile == null)
    {
     WindowGeometry geom = new WindowGeometry();
     Shape3D shape = new Shape3D(geom, app);
     PickTool.setCapabilities(shape, PickTool.INTERSECT_FULL);
     objTrans.addChild(shape);
     this.pickShape3DBG = this;
    }



//---------------------------- initialize behavior

   if(debugThis)Logger.debug.println("DEBUG: [Window2D] initPicking...");
   Window2DExamineViewerBehavior behavior = new Window2DExamineViewerBehavior(this, objTrans, canvas, ctl, app, pickShape3DBG);
   behavior.setSchedulingBounds(bounds);
   windowBG.addChild(behavior);


  }

 private boolean setMaterial(Object o, Appearance app)
 {
  if(debugThis)Logger.debug.println("DEBUG: [Window2D.setMaterial] o.getClass=["+o.getClass()+"]");
  if(o instanceof TransformGroup)
    {
     TransformGroup tg = (TransformGroup)o;
     java.util.Enumeration enum =  tg.getAllChildren();
     int i=0;
     while(enum.hasMoreElements())
       {
        Object o2 = enum.nextElement();
        if(setMaterial(o2, app))
          {  //replace the TransformGroup with a BranchGroup
           BranchGroup bg = new BranchGroup();
           pickShape3DBG = bg;
           tg.removeChild(i);
           bg.addChild(((Shape3D)o2));
           tg.addChild(bg);
          }
        i++;
       }
     return(false);
    }

  if(o instanceof Shape3D)
    {  // set appearance and texture-coordinate-generation
     Shape3D shape = (Shape3D)o;
     shape.setAppearance(app);
     app.setTexCoordGeneration(new TexCoordGeneration(TexCoordGeneration.OBJECT_LINEAR, TexCoordGeneration.TEXTURE_COORDINATE_2));
     PickTool.setCapabilities(shape, PickTool.INTERSECT_FULL);
     return(true);
    }
  Logger.warning.println("WARNING: [Window2D.setMaterial] o has unknown class o.getClass=["+o.getClass()+"]");
  return(false);
 }

 static Matrix4f getTextureTransformMatrix()
 {
  try
    {
     String str = System.getProperty("jaugment2.UI.java3dUI.window2D.Window2D.textureMatrix");
     if(str == null)
       return(null);
     float floats[] = new float[4*4];
     java.util.StringTokenizer st = new java.util.StringTokenizer(str, ",");
     int i=0;
     while(st.hasMoreElements())
       {
        String val = st.nextToken();
        floats[i] = Float.parseFloat(val);
        i++;
       }
     return(new Matrix4f(floats));
    }
  catch(Exception e)
    {
     e.printStackTrace(Logger.err);
    }
  return(null);
 }

 static Matrix3f getMouseTransformMatrix()
 {
  try
    {
     String str = System.getProperty("jaugment2.UI.java3dUI.window2D.Window2D.mouseMatrix");
     if(str == null)
       {
        if(debugThis)Logger.debug.println("DEBUG: [Window2D] getMouseTransformMatrix() jaugment2.UI.java3dUI.window2D.Window2D.mouseMatrix==null");
        return(null);
       }
     if(debugThis)Logger.debug.println("DEBUG: [Window2D] getMouseTransformMatrix() jaugment2.UI.java3dUI.window2D.Window2D.mouseMatrix != null");
     float floats[] = new float[3*3];
     java.util.StringTokenizer st = new java.util.StringTokenizer(str, ",");
     int i=0;
     while(st.hasMoreElements())
       {
        String val = st.nextToken();
        floats[i] = Float.parseFloat(val);
        i++;
       }
     return(new Matrix3f(floats));
    }
  catch(Exception e)
    {
     e.printStackTrace(Logger.err);
    }
  return(null);
 }


 private BranchGroup pickShape3DBG = null;
}