/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Java3D-UI
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2000-12-13, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  *       - normalizing hit-coordinates works
  *       - testcode for mouse-click-delivery to component (not working)
  * 0.0.2 (2000-12-13, Marcus Wolschon<Suran@gmx.net>)
  *       - clicking is working
  *       - clean up
  *       - better documentation
  * 0.0.3 (2000-12-14, Marcus Wolschon<Suran@gmx.net>)
  *       - some sanity-checks
  *       - The component itself is set as the source of the event
  * 0.0.4 (2000-12-15, Marcus Wolschon<Suran@gmx.net>)
  *       - uses getComponentAt recursively
  * 0.0.5 (2000-12-22, Marcus Wolschon<Suran@gmx.net>)
  *       - reacts to mouse pressed and -released
  *       - updateGraphics()
  *       - focus_b
  * -------------------------------------------------------------------
  * TODO:
  */
package jaugment2.UI.java3dUI.window2D;

  //JDK
import java.awt.*;
import java.awt.Component;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
  //Java3D
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.picking.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.image.TextureLoader;
  //project
import jaugment2.helpers.Logger;

/**
 * Behavior, that processes, translates and deliveres mouse-events for a java.awt.Component
 * and updates a Texture with a view of that component
 */
public class Window2DBehavior extends Behavior
{

  float size;
  PickCanvas pickCanvas;
  PickResult[] pickResult;

  /**
   * The component we process and translate the mouse-events for
   */
  private Component ctl;

  /**
   * The component, that has the keyboard-focus
   */
  public static Component focus_c;
  /**
   * The behavior for the component, that has the keyboard-focus
   */
  public static Window2DBehavior focus_b;

 /**
  * Recreate and draw the texture
  */
 public void updateGraphics()
 {
  if(ctl.getWidth()>0 && ctl.getHeight()>0)
    {
     BufferedImage img = new BufferedImage(ctl.getWidth(), ctl.getHeight(), BufferedImage.TYPE_INT_RGB);
     Graphics ctlg = img.getGraphics();
     ctl.paintAll(ctlg);
//     ctlg.drawLine(0,0, 59, 25);
//     ctlg.drawLine(0, 25, 59, 0);
     Texture txt = (new TextureLoader(img)).getTexture();
     app.setTexture(txt);
    }
  else
   {
    Logger.warning.println("WARNING: [Window2DBehavior] control has size 0");
   }

 }

  /**
   * The apearance to render the texture with the view of the component into
   */
  private Appearance app;

  public Window2DBehavior(Canvas3D canvas3D, BranchGroup branchGroup, float size, Component ctl, Appearance app)
  {
   this.ctl = ctl;
   this.focus_c = ctl;
   this.focus_b = this;
   this.app = app;
   pickCanvas = new PickCanvas(canvas3D, branchGroup);
//   pickCanvas.setTolerance(5.0f);
   pickCanvas.setTolerance(0.1f);
   pickCanvas.setMode(PickCanvas.GEOMETRY_INTERSECT_INFO);
   this.size = size;
  }

  private static int mouse_mask = MouseEvent.MOUSE_PRESSED ;//|  ;//| MouseEvent.MOUSE_PRESSED | MouseEvent.MOUSE_RELEASED | MouseEvent.MOUSE_DRAGGED;
  private static int mouse_mask2 =  MouseEvent.MOUSE_RELEASED;

  public void initialize()
  {
   wakeupOn (new WakeupOnAWTEvent(mouse_mask));
  }

  public void processStimulus (Enumeration criteria)
  {
   boolean second = false;
    WakeupCriterion wakeup;

    while (criteria.hasMoreElements())
     {
      wakeup = (WakeupCriterion) criteria.nextElement();
      if (wakeup instanceof WakeupOnAWTEvent)
       {
	AWTEvent[] events = ((WakeupOnAWTEvent)wakeup).getAWTEvent();

	for (int i=0; i<events.length; i++)
         {
          int eventId = events[i].getID();

          System.err.println("DEBUG: [Window2DBehavior] event");

           if(events[i] instanceof KeyEvent)
           {
            System.err.println("DEBUG: [Window2DBehavior] key typed");
            focus_c.dispatchEvent(events[i]);
           }
          else
//	  if( (eventId & mouse_mask) != 0)
           if(events[i] instanceof MouseEvent)
           {

            Logger.debug.println("DEBUG: [Window2DBehavior] mouse-event");
            MouseEvent me2 = (MouseEvent)events[i];
            second =  (me2.getID() == MouseEvent.MOUSE_PRESSED);
            int pos[] = getClickPosition( me2 );
            if(pos == null)
              continue;

            int _x = pos[0];
            int _y = pos[1];




	     System.out.println ("denormalized Coordinates of intersection pt: "+_x+" : "+_y);

             Component ctl2       = ctl;
             Component parent_ctl = ctl;
             boolean changed = true;

             while(changed)
               {
                ctl2 = parent_ctl.getComponentAt(_x, _y);
                if(ctl2==null)
                  {
                   Logger.err.println ("ERROR: [Window3DBehavior] no combonent at "+_x+" : "+_y);
                   ctl2 = ctl.locate(_x, _y);
                   if(ctl2==null)
                      Logger.err.println ("ERROR: [Window3DBehavior] not within combonent at "+_x+" : "+_y);
                  }
                else
                  {
                   Logger.debug.println("DEBUG: [Window3DBehavior] ctl at "+_x+" : "+_y+" is a ["+ctl2.getClass().getName()+"]");
                  }


                if(ctl2 != null)
                  {
                   focus_c = ctl2;
                   focus_b = this;
                   _x = _x- ctl2.getX();
                   _y = _y- ctl2.getY();
                   changed = (ctl2 != parent_ctl);
                   parent_ctl = ctl2;
                  }
                else
                  {
                   ctl2 = ctl;
                   changed = false;
                  }
               }


             MouseEvent me = new MouseEvent( ctl2/* me2.getComponent()*/, me2.getID(), me2.getWhen(), me2.getModifiers(),
                                            _x, _y , me2.getClickCount(), me2.isPopupTrigger());


             ctl2.dispatchEvent(me);

                // leave a mark where we clicked
/*                Graphics  g =  ctl.getGraphics();
                g.setColor(Color.black);
                g.fillRect(_x,_y,14,14);*/

               // update the texture
               //TODO: make this faster
//----

/*            if(ctl.getWidth()>0 && ctl.getHeight()>0)
              {
               ctl.setSize(100, 100);
              }*/

            updateGraphics();
/*            if(ctl.getWidth()>0 && ctl.getHeight()>0)
              {
               BufferedImage img = new BufferedImage(ctl.getWidth(), ctl.getHeight(), BufferedImage.TYPE_INT_RGB);
               Graphics ctlg = img.getGraphics();
               ctl.paintAll(ctlg);
               ctlg.drawLine(0,0, 59, 25);
               ctlg.drawLine(0, 25, 59, 0);
               Texture txt = (new TextureLoader(img)).getTexture();
               app.setTexture(txt);
              }
            else
              {
               Logger.warning.println("WARNING: [Window2DBehavior] control has size 0");
              }*/
//----
            System.err.println("WARNING: [Window2DBehavior] done");
	    }
	  }
	}
      }

     wakeupOn (new WakeupOnAWTEvent(second?mouse_mask2:mouse_mask));
     System.err.println("WARNING: [Window2DBehavior] return");
    }



  private int[] getClickPosition(MouseEvent evt)
  {
   System.err.println("WARNING: [Window2DBehavior] getClickPosition");
   int x = evt.getX();
   int y = evt.getY();
   pickCanvas.setShapeLocation(x, y);

   Point3d eyePos = pickCanvas.getStartPosition();
   pickResult = pickCanvas.pickAllSorted();//TODO: catch exception if we picked the small boxen and ignore the click
   if(pickResult == null)
     {
      System.err.println("WARNING: [Window2DBehavior] pickResult == null");
      return(null);
     }


   // Get closest intersection results
   PickIntersection pi = pickResult[0].getClosestIntersection(eyePos);

   GeometryArray curGeomArray = pi.getGeometryArray();

   // Position sphere at intersection point
   Vector3d v = new Vector3d();
   Point3d intPt = pi.getPointCoordinatesVW();

   Point3d []ptw = pi.getPrimitiveCoordinatesVW();
   Point3d []pt  = pi.getPrimitiveCoordinates();

   System.out.println ("=============");
   System.out.println ("Coordinates of intersection pt:"+intPt);
   System.out.println ("Coordinates of vertices: ");

   double minx = Double.MAX_VALUE;
   double miny = Double.MAX_VALUE;
   double maxx = Double.MIN_VALUE;
   double maxy = Double.MIN_VALUE;

   for(int k=0;k<pt.length;k++)
      {
       System.out.println (k + ":" + ptw[k].x + " " + ptw[k].y + " " + ptw[k].z);
       if(ptw[k].x<minx)
          minx = ptw[k].x;
       if(ptw[k].y<miny)
          miny = ptw[k].y;
       if(ptw[k].x>maxx)
          maxx = ptw[k].x;
       if(ptw[k].y>maxy)
          maxy = ptw[k].y;
      }

   double normalx = (intPt.x-minx)/(maxx-minx);
   double normaly = (intPt.y-miny)/(maxy-miny);
   System.out.println ("normalized Coordinates of intersection pt: "+normalx+" : "+normaly+" max=["+maxx+":"+maxy+"], min=["+minx+":"+miny+"]");

   if(ctl==null)
     {
      System.err.println("ERROR: [Window2DBehavior] ctl==null");
      return(null);
     }

   int _x = (int)(ctl.getWidth()*normalx);//+ctl.getX();
   int _y = (int)(ctl.getHeight()*(1-normaly));//+ctl.getY();

   Point3d closestVert = pi.getClosestVertexCoordinatesVW();
   System.out.println ("Closest vertex: "+closestVert);


   return(new int[] {_x, _y});
  }
}

