/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: mirror
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-04-05, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * -------------------------------------------------------------------
  * TODO:
  *       - get the URL of the scene to load
  */
package jaugment2.java3d.hand;


  //Swing
import java.io.Serializable;
import java.io.IOException;
import java.rmi.MarshalledObject;
import java.rmi.RemoteException;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Map;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Iterator;
import java.util.Enumeration;
import java.awt.Color;
  //JINI
import net.jini.core.lookup.ServiceItem;
  //serviceui-draft-standart
import net.jini.lookup.entry.UIDescriptor;
import net.jini.lookup.ui.MainUI;
import net.jini.lookup.ui.attribute.UIFactoryTypes;
import net.jini.lookup.ui.attribute.RequiredPackages;
  //Java3D
import javax.media.j3d.BranchGroup;
import com.sun.j3d.utils.geometry.ColorCube;
import com.sun.j3d.loaders.Scene;
import com.sun.j3d.loaders.vrml97.VrmlLoader;
import com.sun.j3d.loaders.vrml97.VrmlScene;
import com.sun.j3d.loaders.vrml97.impl.MovieTexture;
import javax.media.j3d.Behavior;
import javax.media.j3d.QuadArray;
import javax.media.j3d.Appearance;
import javax.media.j3d.Shape3D;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.Material;
import javax.media.j3d.RenderingAttributes;
import javax.media.j3d.ColorInterpolator;
import javax.media.j3d.Alpha;
import javax.media.j3d.RotationInterpolator;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.Sensor;
import javax.media.j3d.InputDevice;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.WakeupOnElapsedFrames;
import javax.vecmath.Color3f;
import javax.vecmath.*;
  //project
import jaugment2.helpers.Logger;
import jaugment2.UI.java3dUI.Java3DUIFactory;



public class HandServiceJava3DRepresentation extends BranchGroup
{
 private static final boolean debugThis = true;



 public class SensorBehavior extends Behavior
 {
  private WakeupOnElapsedFrames conditions = new WakeupOnElapsedFrames(0);
  private TransformGroup transformGroup;
  private Sensor sensor;
  private Transform3D transform = new Transform3D();

  public SensorBehavior( TransformGroup tg, Sensor sensor )
  {
   this.transformGroup = tg;
   this.sensor = sensor;
  }

  public void initialize()
  {
   wakeupOn( conditions );
  }

  public void processStimulus( Enumeration criteria )
  {
   sensor.getRead( transform );
   transformGroup.setTransform( transform );
   wakeupOn( conditions );
  }
 }


 /**
  * Get a UIDescriptor as needed by the serviceui-draft-standart
  */
 public static UIDescriptor getUIDescriptor()
  {
   try
     {
      return(new UIDescriptor(MainUI.ROLE,
                              Java3DUIFactory.TOOLKIT,
                              getServiceUIProperties(),
                              new MarshalledObject(new Factory()))
                             );
     }
   catch(IOException e)
     {
      Logger.err.print(e.getLocalizedMessage());
      e.printStackTrace(Logger.err);
     }
   return(null);
  }

  /**
   * Gets a serviceui-draft-standart-compactible attribute-set of this representation
   */
  static Set getServiceUIProperties()
  {
   HashSet set = new HashSet();
   set.add("javax.media.j3d.BranchGroup");
   set.add("javax.media.j3d.Group");
   set.add("javax.media.j3d.Node");
   set.add("javax.media.j3d.SceneGraphObject");

   HashSet retval = new HashSet();
   retval.add(new UIFactoryTypes(set));

   HashMap map = new HashMap();
   map.put("javax.media.j3d", Package.getPackage("javax.media.j3d").getSpecificationVersion());
   RequiredPackages req = new RequiredPackages(map);
   retval.add(req);

   return(retval);
  }


  /**
   * A serializable factory for creating this UI
   */
  public static class Factory implements Java3DUIFactory, Serializable
  {

    /**
     * Returns a <CODE>BranchGroup</CODE>.
     */
    public BranchGroup getRepresentation(Object roleObject)
    {
     Logger.debug.println("DEBUG: [HandServiceJava3DRepresentation.factory] getRepresentation called");
     try
       {
        net.jini.core.lookup.ServiceItem item = (net.jini.core.lookup.ServiceItem)roleObject;
        return(new HandServiceJava3DRepresentation((HandServiceProxy)item.service));
       }
     catch(Exception e)
       {
        e.printStackTrace(Logger.err);
       }
     return(null);
    }


   public Iterator getInputDevices(BranchGroup bg)
   {
    return(((HandServiceJava3DRepresentation)bg).getInputDevices());
   }

   public void activateRepresentation(BranchGroup bg)
   {
   }


  }

 private static final boolean debutThis = true;

 private Shape3D newCube(float minX, float maxX, float minY, float maxY, float minZ, float maxZ)
 {
   Point3f verts[] = new Point3f[8];
    verts[0] = new Point3f( maxX, maxY, maxZ);
    verts[1] = new Point3f( minX, maxY, maxZ);
    verts[2] = new Point3f( minX, minY, maxZ);
    verts[3] = new Point3f( maxX, minY, maxZ);
    verts[4] = new Point3f( maxX, maxY, minZ);
    verts[5] = new Point3f( minX, maxY, minZ);
    verts[6] = new Point3f( minX, minY, minZ);
    verts[7] = new Point3f( maxX, minY, minZ);

    QuadArray geom = new QuadArray(24, GeometryArray.COORDINATES/* | GeometryArray.TEXTURE_COORDINATE_3*/);

    Point3f pnts[] = new Point3f[24];
    pnts[0] = verts[0];
    pnts[1] = verts[3];
    pnts[2] = verts[7];
    pnts[3] = verts[4];

    pnts[4] = verts[1];
    pnts[5] = verts[5];
    pnts[6] = verts[6];
    pnts[7] = verts[2];

    pnts[8]  = verts[0];
    pnts[9]  = verts[4];
    pnts[10] = verts[5];
    pnts[11] = verts[1];

    pnts[12] = verts[3];
    pnts[13] = verts[2];
    pnts[14] = verts[6];
    pnts[15] = verts[7];

    pnts[16] = verts[0];
    pnts[17] = verts[1];
    pnts[18] = verts[2];
    pnts[19] = verts[3];

    pnts[20] = verts[7];
    pnts[21] = verts[6];
    pnts[22] = verts[5];
    pnts[23] = verts[4];

    geom.setCoordinates(0, pnts);

    return(new Shape3D(geom, app));
 }

 private Appearance app = new Appearance();

 public HandServiceJava3DRepresentation(HandServiceProxy proxy)
 {
  super();

   try
     {
      if(debugThis)Logger.debug.println("DEBUG: [HandServiceJava3DRepresentation] init()");

        //Transform-Group
      TransformGroup trans = new TransformGroup();
                     trans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
      addChild(trans);

        //Apperance

      Appearance app = new Appearance();
      Material mat = new Material(new Color3f(0.0f,0.0f,0.0f), new Color3f(Color.red),new Color3f(0.0f,0.0f,0.0f),new Color3f(0.0f,0.0f,0.0f),110.0f);//ambient, c, dffuse,specular,shininess
      app.setMaterial(mat);
      app.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
      mat.setCapability(Material.ALLOW_COMPONENT_WRITE);
      Alpha a = new Alpha(-1, Alpha.INCREASING_ENABLE,
                          0,0,
                          5000,0,0,
                          0,0,0);
      ColorInterpolator ci = new ColorInterpolator(a, mat, new Color3f(Color.red), new Color3f(Color.blue));
      BoundingSphere bsp = new BoundingSphere(new Point3d(0.0,0.0,0.0), 100.0);
      ci.setSchedulingBounds(bsp);
      addChild(ci);

        //Shape
      float scale = 0.2f/1.5f;
      Shape3D s = newCube( -scale*0.5f, scale*0.5f,
                           -scale*0.2f, scale*0.2f,
                           -scale*0.5f, scale*1.0f
                         );
      trans.addChild(s);

        //Behavior
/*      Class c = proxy.getHandInputClass();
      input = (HandInput)c.newInstance();*/
      input = new HandInputDevice();
      input.initialize();
      //TODO: test

	// Register the VirtualInputDevice with Java 3D
//	u.getViewer().getPhysicalEnvironment().addInputDevice( device );

      SensorBehavior sb = new SensorBehavior(trans, input.getSensor(0) ); //TODO: configurable, maybe AutomataBehavior
      sb.setSchedulingBounds( new BoundingSphere( new Point3d(0.0,0.0,0.0), Float.MAX_VALUE ));
      addChild(sb);
     }
  catch(Exception e)
    {
     e.printStackTrace();
    }

 }

 public Iterator getInputDevices()
 {
  java.util.LinkedList ll = new java.util.LinkedList();
  ll.add(input);
  return(ll.iterator());
 }


 private HandInputDevice/*HandInput*/ input;

}