/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: mirror
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-07-26, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2001-07-29, Marcus Wolschon<Suran@gmx.net>)
  *       - it works with 1 channel
  *       - much less processor-load
  * -------------------------------------------------------------------
  * TODO:
  *        - add more channels
  */
package jaugment2.java3d.hand;

  //Java3D
import javax.vecmath.Vector3f;
import javax.media.j3d.InputDevice;
import java.awt.event.*;
  //Serial port
import javax.comm.*;
import java.io.*;
import java.util.Enumeration;
import java.util.TooManyListenersException;


import javax.vecmath.Vector3f;
import javax.media.j3d.InputDevice;

/**
 * Title:        JAugment II - Java3d-UI
 * Description:  Jini-Based system of wearable-optimized services
 * Copyright:    Copyright (c) 1999
 * Company:
 * @author Marcus Wolschon Suran@gmx.net
 * @version
 */

public class SerialHandInput implements HandInput
{

 private SerialPortListener listener = null;
 private static final boolean debugThis = true;

  public SerialHandInput()
  {
   if(debugThis) jaugment2.helpers.Logger.debug.println("SerialHandInput()");
   listener = new SerialPortListener(position);
  }





 private class meanFilter
 {
  private static final int samples = 200;
  private int values[] = new int[samples];
  private int pos = 0;

  public int getMean()
  {
   long retval = 0;
   for(int i=0; i<samples; i++)
     retval+=values[i];
   return( (int)(retval/samples) );
  }

  public void addValue(int v)
  {
   if(pos>(samples-1))
     pos=0;
   values[pos]=v;
   pos++;
  }

 }


  private class SerialPortListener  implements javax.comm.SerialPortEventListener, java.lang.Runnable
  {
  private  CommPortIdentifier portId;
  private  SerialPort sPort;
  private  OutputStream os;
  private  InputStream is;

  private Thread t = null;
  private Vector3f position;

  public SerialPortListener(Vector3f position)
  {
   this.position = position;
   t = new Thread(this);
   t.start();
  }


  public void run()
  {
   try
     {
      try
        {
         portId = CommPortIdentifier.getPortIdentifier("/dev/ttyS0");
         sPort = (SerialPort)portId.open("SerialTextInputDevice", 30000);
        }
      catch(javax.comm.NoSuchPortException e4)
        {
         portId = (CommPortIdentifier)CommPortIdentifier.getPortIdentifiers().nextElement();
         jaugment2.helpers.Logger.debug.println("portId = "+portId.getName());
//         CommPortIdentifier.addPort("/dev/ttyS0", CommPortIdentifier.PORT_SERIAL);
//         portId = CommPortIdentifier.getPortIdentifier("/dev/ttyS0");
         sPort = (SerialPort)portId.open("SerialTextInputDevice", 30000);
        }
      catch(java.lang.NullPointerException e5)
        {//
//         CommPortIdentifier.loadDriver("com.sun.comm.SolarisDriver");
//         portId = CommPortIdentifier.getPortIdentifier("/dev/ttyS0");
//         sPort = (SerialPort)portId.open("SerialTextInputDevice", 30000)
         sPort = (SerialPort)(new com.sun.comm.SolarisDriver()).getCommPort("/dev/ttyS0", CommPortIdentifier.PORT_SERIAL);
        }


      sPort.setSerialPortParams(115200,
                                SerialPort.DATABITS_8,
                                SerialPort.STOPBITS_1,
                                SerialPort.PARITY_NONE
                               );


      os = sPort.getOutputStream();
      is = sPort.getInputStream();


      int mean = 0;
      int old_mean = 0;
      int mean_cnt = 0;

      meanFilter mf = new meanFilter();

      int val = 0;
      int val0 = 0;
      int val1 = 0;
      int val2 = 0;
      int state=0;
      String frame="Val( - - )"+((char)10)+((char)13);
      while(true)
       {
        int r = is.read();
        if(r != -1)
          {
           switch(state)
            {
             case 0:
             case 1:
             case 2:
             case 3:
             case 5:
             case 7:
             case 9:
             case 10:
              {
               if(r==frame.charAt(state))
                 {
                  state++;
                 }
               else
                {
                 state=0;
                }
              break;
             }
           case 4:
             {
              val0=r;
              val = r;
              state++;
              break;
             }
           case 6:
             {
              val1=r;
              val = val + (r<<8);
              state++;
              break;
             }
           case 8:
             {
              val2=r;
              state++;
              break;
             }
           case 11:
             {
              String s=null;

              mean += val;
              mean_cnt++;
              if(mean_cnt==150)
                {
                 old_mean= mean/75;
                 mean_cnt=0;
                 mean=0;
                }

              mf.addValue(val);

/*              s = java.lang.Integer.toString(val1,16);
              for(int i=s.length(); i<3;i++)
                  s+=" ";
              System.out.print("val=[0x"+s+":");

              s = java.lang.Integer.toString(val0,16);
              for(int i=s.length(); i<3;i++)
                  s+=" ";
              System.out.print("0x"+s+"]");

              s = java.lang.Integer.toString(val2,16);
              for(int i=s.length(); i<3;i++)
                  s+=" ";
              System.out.print("seq=[0x"+s+"]");

              s = java.lang.Integer.toString(old_mean,16);
              for(int i=s.length(); i<3;i++)
                  s+=" ";
              System.out.print("mean=[0x"+s+"]");

              s = java.lang.Integer.toString(val,16);
              for(int i=s.length(); i<6;i++)
                  s+=" ";
              System.out.print("0x"+s);*/

              float y = ((float)mf.getMean()-0x200)/30;
              if(y < position.y*0.95 || y > position.y*1.05 )
//              if(y != position.y)
                {
                 position.set(position.x, y , position.z);

                 for(int i=0; i<y*(30/5); i++)
                   System.out.print("*");

                 System.out.println(""+y);
                }

             }
           default:
             state=0;
          }



/*    if(r==10)
      {}
    else
    if(r==13)
      {
       for(int i=0; i<val/100; i++)
         System.out.print("*");
       System.out.println("");
       val=0;
       state=0;
      }
    else
      {
       if(last==40 || last==45)
         {
          String s = java.lang.Integer.toString(r,10);
          for(int i=s.length(); i<3;i++)
            s+=" ";
          System.out.print("0x"+s);
          if(state==0)
            {
             val = r;
             state=1;
            }
          else
          if(state==1)
            {
             val = val + 512*r;
             state=2;
            }
         }
       else
         {System.out.print(""+(char)r);}
      }
    last = r;  */
   }
}

/*     sPort.addEventListener(this);
     sPort.notifyOnDataAvailable(true);
     sPort.enableReceiveTimeout(30);
     System.out.println("ready");
*/    }
   catch(Exception ex)
      {
       ex.printStackTrace();
       return;
      }
  }




  /** Some data arived on the serial-port
    */
  public void serialEvent(SerialPortEvent e)
  {
   System.err.println("DEBUG: serialEvent ["+e.getClass().getName()+"]");
   // Create a StringBuffer and int to receive input data.
   StringBuffer inputBuffer = new StringBuffer();
   int newData = 0;

   // Determine type of event.
   switch (e.getEventType())
   {
    // Read data until -1 is returned. If \r is received substitute
    // \n for correct newline handling.
    case SerialPortEvent.DATA_AVAILABLE:
      {
        while (newData != -1)
        {
         try
           {
            newData = is.read();
            if(newData == -1)
              {
               break;
              }
            if('\r' == (char)newData)
              {
               System.out.println("");
              }
            else
              {
               System.out.print(""+((char)newData));
              }
           }
         catch(IOException ex)
           {
            ex.printStackTrace();
            return;
          }
        }
      break;
     }

   }

  }

  };





  Vector3f position = new Vector3f();
  public void getHandPosition(Vector3f pos )
  {
   pos.set(this.position);
  }
  public void setHandPosition(Vector3f pos )
  {
   this.position.set(pos);
  }


  private InputDevice device;
  public void setDevice(InputDevice device)
  {
   this.device = device;
  }


  private float step_rate = 0.0025f;
  private long time_last_state_change = System.currentTimeMillis();
  public void setStepRate( float stepRate )
  {
   this.step_rate = stepRate;
  }




  public void close()
  {

  }
}