/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: mirror
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-04-05, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * -------------------------------------------------------------------
  * TODO:
  *        - implement it
  *        - implement the interface HandInput
  *        - implement device and sensor
  *        - use both
  */
package jaugment2.java3d.hand;

  //SWING
import java.awt.*;
import javax.swing.*;
  //Java3D
import javax.vecmath.Vector3f;
import javax.media.j3d.InputDevice;
import java.awt.event.*;


/**
 * Frame for emulating a data-glove
 */
public class VirtualHandInput extends JFrame implements HandInput
{
  private interface MODES
    {
     static final int STILL=0;
     static final int UP=2;
     static final int DOWN=4;
     static final int LEFT=8;
     static final int RIGHT=16;
     static final int FORWARD=32;
     static final int BACK=64;
    }

  private int move_mode = MODES.STILL;
  Vector3f old_position = new Vector3f();
  Vector3f position = new Vector3f();
  public void getHandPosition(Vector3f pos )
  {
   calculateMotion();
   pos.set(this.position);
  }

  public void setHandPosition(Vector3f pos )
  {
   this.position.set(pos);
  }

  private InputDevice device;
  public void setDevice(InputDevice device)
  {
   this.device = device;
  }

  private float step_rate = 0.0025f;
  private long time_last_state_change = System.currentTimeMillis();
  public void setStepRate( float stepRate )
  {
   this.step_rate = stepRate;
  }


  BorderLayout borderLayout1 = new BorderLayout();
  JButton jButton_up = new JButton();
  JButton jButton_down = new JButton();
  JButton jButton_right = new JButton();
  JButton jButton_left = new JButton();
  JPanel jPanel1 = new JPanel();
  JButton jButton_go = new JButton();
  JButton jButton_back = new JButton();
  JButton jButton_still = new JButton();
  BorderLayout borderLayout2 = new BorderLayout();

  public void close()
  {
   this.hide();
  }

  public VirtualHandInput()
  {
    try
    {
      jbInit();
      this.validate();
      this.pack();
      this.show();
      this.setVisible(true);
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }
  private void jbInit() throws Exception
  {
    jButton_up.setText("up");
    jButton_up.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mousePressed(MouseEvent e)
      {
        jButton_up_actionPerformed(e);
      }
      public void mouseReleased(MouseEvent e)
      {
        onMouseReleased(e);
      }
    });
    this.getContentPane().setLayout(borderLayout1);
    jButton_down.setText("down");
    jButton_down.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mousePressed(MouseEvent e)
      {
        jButton_down_actionPerformed(e);
      }
      public void mouseReleased(MouseEvent e)
      {
        onMouseReleased(e);
      }
    });
    jButton_right.setText("right");
    jButton_right.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mousePressed(MouseEvent e)
      {
       jButton_right_actionPerformed(e);
      }
      public void mouseReleased(MouseEvent e)
      {
        onMouseReleased(e);
      }
    });
    jButton_left.setText("left");
    jButton_left.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mousePressed(MouseEvent e)
      {
        jButton_left_actionPerformed(e);
      }
      public void mouseReleased(MouseEvent e)
      {
        onMouseReleased(e);
      }
    });
    jButton_go.setText("go");
    jButton_go.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mousePressed(MouseEvent e)
      {
       jButton_go_actionPerformed(e);
      }
      public void mouseReleased(MouseEvent e)
      {
       onMouseReleased(e);
      }
    });
    jButton_back.setText("back");
    jButton_back.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mousePressed(MouseEvent e)
      {
       jButton_back_actionPerformed(e);
      }

      public void mouseReleased(MouseEvent e)
      {
       onMouseReleased(e);
      }
    });
    jButton_still.setText("stop");
    jButton_still.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mousePressed(MouseEvent e)
      {
        jButton_still_actionPerformed(e);
      }
      public void mouseReleased(MouseEvent e)
      {
        onMouseReleased(e);
      }
    });
    jPanel1.setLayout(borderLayout2);
    this.getContentPane().add(jButton_up, BorderLayout.NORTH);
    this.getContentPane().add(jButton_down, BorderLayout.SOUTH);
    this.getContentPane().add(jButton_right, BorderLayout.EAST);
    this.getContentPane().add(jButton_left, BorderLayout.WEST);
    this.getContentPane().add(jPanel1, BorderLayout.CENTER);
    jPanel1.add(jButton_go, BorderLayout.NORTH);
    jPanel1.add(jButton_still, BorderLayout.CENTER);
    jPanel1.add(jButton_back, BorderLayout.SOUTH);
  }

  private void calculateMotion()
   {
    long current_time = System.currentTimeMillis();
    long elapsed_time = current_time - time_last_state_change;

     switch(move_mode)
      {
       case MODES.STILL:
            break;
       case MODES.LEFT:
            position.x = old_position.x - step_rate*elapsed_time;
            break;
       case MODES.RIGHT:
            position.x = old_position.x + step_rate*elapsed_time;
            break;
       case MODES.UP:
            position.y = old_position.y + step_rate*elapsed_time;
            break;
       case MODES.DOWN:
            position.y = old_position.y - step_rate*elapsed_time;
            break;
       case MODES.FORWARD:
            position.z = old_position.z - step_rate*elapsed_time;
            break;
       case MODES.BACK:
            position.z = old_position.z + step_rate*elapsed_time;
            break;
       default:
            {
             throw new java.lang.IllegalStateException("Unknown value for move_mode");
            }
        }
    }


  void jButton_up_actionPerformed(Object e)
  {
   if(move_mode != MODES.UP)
     {
      time_last_state_change =  System.currentTimeMillis();
      move_mode = MODES.UP;
      old_position.set(position);
     }
  }

  void jButton_down_actionPerformed(Object e)
  {
   if(move_mode != MODES.DOWN)
     {
      time_last_state_change =  System.currentTimeMillis();
      move_mode = MODES.DOWN;
      old_position.set(position);
     }
  }

  void jButton_right_actionPerformed(Object e)
  {
   if(move_mode != MODES.RIGHT)
     {
      time_last_state_change =  System.currentTimeMillis();
      move_mode = MODES.RIGHT;
      old_position.set(position);
     }
  }

  void jButton_left_actionPerformed(Object e)
  {
   if(move_mode != MODES.LEFT)
     {
      time_last_state_change =  System.currentTimeMillis();
      move_mode = MODES.LEFT;
      old_position.set(position);
     }
  }

  void jButton_go_actionPerformed(Object e)
  {
   if(move_mode != MODES.FORWARD)
     {
      time_last_state_change =  System.currentTimeMillis();
      move_mode = MODES.FORWARD;
      old_position.set(position);
     }
  }

  void jButton_back_actionPerformed(Object e)
  {
   if(move_mode != MODES.BACK)
     {
      time_last_state_change =  System.currentTimeMillis();
      move_mode = MODES.BACK;
      old_position.set(position);
     }
  }

  void jButton_still_actionPerformed(Object e)
  {
   if(device != null)
     device.setNominalPositionAndOrientation();
  }

  void onMouseReleased(Object e)
  {
   this.move_mode = MODES.STILL;

  }



}