/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: mirror
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2000-12-23, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * -------------------------------------------------------------------
  * TODO:
  */
package jaugment2.java3d.mirror;

  //JDK
import java.rmi.RemoteException;
import java.rmi.Remote;
import java.rmi.MarshalledObject;
import java.util.StringTokenizer;
import java.util.LinkedList;
import java.util.Hashtable;
import java.util.Date;
import java.util.Enumeration;
import java.net.URL;
import java.net.InetAddress;
import java.lang.Runnable;
import java.lang.Process;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.LineNumberReader;
  //JINI
import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.UnknownLeaseException;
import net.jini.core.lease.Lease;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;

import net.jini.core.entry.Entry;
import net.jini.core.discovery.LookupLocator;
import net.jini.discovery.DiscoveryListener;
import net.jini.discovery.DiscoveryEvent;
  //project
import jaugment2.jini.simpleService.simpleService;
import jaugment2.jini.SimpleLandlord;
import jaugment2.jini.SimpleLandlord.LeaseListener;
import jaugment2.jini.SimpleLandlord.EventRegistration;
import jaugment2.helpers.Logger;
import jaugment2.UI.java3dUI.Java3DUIFactory;


//helper-mark for configure to determine that rmic needs to be executed for this file
//rmimakehelper


/**
 * Service providing a Java3D-Mirror
 */
public class MirrorService extends simpleService implements Remote
{

  //--------------------- initiaslisation&administration ------------------------------------------------------//

  private MirrorServiceProxy myproxy;
  private String lastmsg = null;


  /**
   * Load our state (e.g. attrubutes, service-id) from persistent storage
   */
  private void loadMe()
  {
      String filename = System.getProperty("jaugment2.java3d.mirror.MirrorService.persistentStateFilename");
      if(filename==null)
        {
         filename = "MirrorServiceState.dat";
         System.setProperty("jaugment2.java3d.mirror.MirrorService.persistentStateFilename", filename);
        };

      persistentState.setProperty("persistentStateFilename", filename);

   load();


   if(persistentState.get("attr")==null)
     {
      Entry[] attrSets = new Entry[]
                   {
                     new net.jini.lookup.entry.Name("MirrorService"),
                     new net.jini.lookup.entry.Comment("a service providing a Java3D-mirror"),
                     new net.jini.lookup.entry.ServiceInfo("MirrorService", "Marcus Wolschon <Suran@gmx.net>", "Marcus Wolschon <Suran@gmx.net>", "0.0.1", "standart", "00000"),

                     MirrorServiceJava3DRepresentation.getUIDescriptor()
                    };
      persistentState.put("attr", attrSets);
     };

   save();
  };


  public MirrorService() throws RemoteException
  {
   this(null);
  };

  public MirrorService(LookupLocator locallocator) throws RemoteException
  {
   Logger.debug.println("DEBUG: [MirrorService] creating myself...");
   try
     {

      persistentState.setProperty("groups", "public, Suran");
      persistentState.setProperty("locators", "jini://localhost, jini://127.0.0.1");

      myproxy = new MirrorServiceProxy(this);

      loadMe();


      Entry[] attrSets = (Entry[])persistentState.get("attr");

      this.init(myproxy, attrSets, this, locallocator);
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     };
  };



 /** restart this service once it has been stopped
   * @see stop
   */
 public void restart() throws Exception
 {
 }

 /** stop this service. If exit is false all references to this
   * service are still valid but it has deregistered everywhere.
   * (this includes the RMI-activation-daemon)
   * @see restart
   * @see destroy
   * @param exit stop running
   */
 public void stop(boolean exit) throws Exception
 {
 }

 /** stop this service, destroy any persistent storage and exit
   */
 public void destroy() throws RemoteException
 {
  deinit();
 }
}