/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: mirror
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2000-12-23, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * -------------------------------------------------------------------
  * TODO:
  */
package jaugment2.java3d.mirror;


  //Swing
import java.io.Serializable;
import java.io.IOException;
import java.rmi.MarshalledObject;
import java.rmi.RemoteException;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Map;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Iterator;
import java.awt.Color;
  //JINI
import net.jini.core.lookup.ServiceItem;
  //serviceui-draft-standart
import net.jini.lookup.entry.UIDescriptor;
import net.jini.lookup.ui.MainUI;
import net.jini.lookup.ui.attribute.UIFactoryTypes;
import net.jini.lookup.ui.attribute.RequiredPackages;
  //Java3D
import javax.media.j3d.BranchGroup;
import com.sun.j3d.utils.geometry.ColorCube;
import com.sun.j3d.loaders.Scene;
import com.sun.j3d.loaders.vrml97.VrmlLoader;
import com.sun.j3d.loaders.vrml97.VrmlScene;
import com.sun.j3d.loaders.vrml97.impl.MovieTexture;
import javax.media.j3d.QuadArray;
import javax.media.j3d.Appearance;
import javax.media.j3d.Shape3D;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.Material;
import javax.media.j3d.RenderingAttributes;
import javax.media.j3d.ColorInterpolator;
import javax.media.j3d.Alpha;
import javax.media.j3d.RotationInterpolator;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.TransformGroup;
import javax.vecmath.Color3f;
import javax.vecmath.*;
  //project
import jaugment2.helpers.Logger;
import jaugment2.UI.java3dUI.Java3DUIFactory;



public class MirrorServiceJava3DRepresentation extends BranchGroup
{


 /**
  * Get a UIDescriptor as needed by the serviceui-draft-standart
  */
 public static UIDescriptor getUIDescriptor()
  {
   try
     {
      return(new UIDescriptor(MainUI.ROLE,
                              Java3DUIFactory.TOOLKIT,
                              getServiceUIProperties(),
                              new MarshalledObject(new Factory()))
                             );
     }
   catch(IOException e)
     {
      Logger.err.print(e.getLocalizedMessage());
      e.printStackTrace(Logger.err);
     }
   return(null);
  }

  /**
   * Gets a serviceui-draft-standart-compactible attribute-set of this representation
   */
  static Set getServiceUIProperties()
  {
   HashSet set = new HashSet();
   set.add("javax.media.j3d.BranchGroup");
   set.add("javax.media.j3d.Group");
   set.add("javax.media.j3d.Node");
   set.add("javax.media.j3d.SceneGraphObject");

   HashSet retval = new HashSet();
   retval.add(new UIFactoryTypes(set));

   HashMap map = new HashMap();
   map.put("javax.media.j3d", Package.getPackage("javax.media.j3d").getSpecificationVersion());
   RequiredPackages req = new RequiredPackages(map);
   retval.add(req);

   return(retval);
  }


  /**
   * A serializable factory for creating this UI
   */
  public static class Factory implements Java3DUIFactory, Serializable
  {

    /**
     * Returns a <CODE>BranchGroup</CODE>.
     */
    public BranchGroup getRepresentation(Object roleObject)
    {
     Logger.debug.println("DEBUG: [MirrorServiceJava3DRepresentation.factory] getRepresentation called");
     try
       {
        return(new MirrorServiceJava3DRepresentation());
       }
     catch(Exception e)
       {
        e.printStackTrace(Logger.err);
       }
     return(null);
    }

   public Iterator getInputDevices(BranchGroup bg)
   {return(null);};

   public void activateRepresentation(BranchGroup bg)
   {};


  }

 private static final boolean debutThis = true;

 public MirrorServiceJava3DRepresentation()
 {
  super();
//  addChild(new ColorCube(0.4));

   try
     {
      VrmlLoader loader = new VrmlLoader();
      Scene s = loader.load("Padd.wrl");
      java.util.Hashtable namedObjects = s.getNamedObjects();
      if(debutThis)
        {
         Logger.debug.println("DEBUG: [MirrorServiceJava3DRepresentation] scene has "+namedObjects.size()+" names objects");
         java.util.Enumeration enum = namedObjects.keys();
         while(enum.hasMoreElements())
           {
            Object o2 = enum.nextElement();
            Logger.debug.println("DEBUG: [MirrorServiceJava3DRepresentation] scene has object named \""+o2+"\"");
           }
        }
      if(namedObjects.containsKey("Screen"))
        {
         Object o = namedObjects.get("Screen");
         if(debutThis)Logger.debug.println("DEBUG: [MirrorServiceJava3DRepresentation] scene has an object \"Screen\" of type ["+o.getClass().toString()+"]");
         if(o instanceof TransformGroup)
           {
            TransformGroup g = (TransformGroup)o;
            java.util.Enumeration enum2 = g.getAllChildren();
            while(enum2.hasMoreElements())
              {
               Object o2 = enum2.nextElement();
               Logger.debug.println("DEBUG: [MirrorServiceJava3DRepresentation] Screen has child of type \""+o2.getClass()+"\"");
               if(o2 instanceof Shape3D)
                 {

                  Appearance app = new Appearance();
                  Material mat = new Material(new Color3f(Color.green),//new Color3f(0.0f,0.0f,0.0f), //ambient
                                              new Color3f(0.0f,0.0f,0.0f),//new Color3f(Color.red), //emissive
                                              new Color3f(Color.red),//new Color3f(0.0f,0.0f,0.0f), //diffuse
                                              new Color3f(Color.blue),//new Color3f(0.0f,0.0f,0.0f), //specular
                                              110.0f);//ambient, c, dffuse,specular,shininess
                  mat.setLightingEnable(true);
                  app.setMaterial(mat);
                  app.setCapability(Appearance.ALLOW_MATERIAL_WRITE);

                  javax.media.j3d.ColoringAttributes ca = app.getColoringAttributes();
                  if(ca==null)
                    ca = new javax.media.j3d.ColoringAttributes();
                  ca.setShadeModel(javax.media.j3d.ColoringAttributes.SHADE_GOURAUD);
                  app.setColoringAttributes(ca);

                  mat.setCapability(Material.ALLOW_COMPONENT_WRITE);
//TODO: lights do not work

                  Shape3D shape = (Shape3D)o2;
                  shape.setAppearance(app);

                  if(debutThis)Logger.debug.println("DEBUG: [MirrorServiceJava3DRepresentation] Screen has the bounds: \""+shape.getBounds()+"\"");

                 }
              }
           }
        }

      addChild(s.getSceneGroup());
      javax.media.j3d.Light pl = new javax.media.j3d.AmbientLight();
      pl.setEnable(true);
      pl.setColor(new Color3f(java.awt.Color.white));
      pl.addScope(this);
//      pl.setPosition(0.0f, 2.0f, 0.0f);
//      javax.media.j3d.Transform3D tr = new javax.media.j3d.Transform3D();
//      tr.setTranslation(new Vector3f(0.0f, 2.0f, 0.0f));
  //    TransformGroup tg = new TransformGroup(tr);
    //  tg.addChild(pl);
     // addChild(tg);
     addChild(pl);
      return;
     }
  catch(Exception e)
    {
     e.printStackTrace();
    }



   Point3f verts[] = new Point3f[8];
    verts[0] = new Point3f(0.5f, 0.5f,  0.5f);
    verts[1] = new Point3f(-1.0f, 0.5f, 0.5f);
    verts[2] = new Point3f(-1.0f,-1.0f, 0.5f);
    verts[3] = new Point3f( 0.5f,-1.0f, 0.5f);
    verts[4] = new Point3f( 0.5f, 0.5f,-0.001f);
    verts[5] = new Point3f( -1.0f, 0.5f,-0.001f);
    verts[6] = new Point3f( -1.0f,-1.0f,-0.001f);
    verts[7] = new Point3f( 0.5f,-1.0f,-0.001f);

    QuadArray geom = new QuadArray(24, GeometryArray.COORDINATES/* | GeometryArray.TEXTURE_COORDINATE_3*/);

    Point3f pnts[] = new Point3f[24];
    pnts[0] = verts[0];
    pnts[1] = verts[3];
    pnts[2] = verts[7];
    pnts[3] = verts[4];

    pnts[4] = verts[1];
    pnts[5] = verts[5];
    pnts[6] = verts[6];
    pnts[7] = verts[2];

    pnts[8]  = verts[0];
    pnts[9]  = verts[4];
    pnts[10] = verts[5];
    pnts[11] = verts[1];

    pnts[12] = verts[3];
    pnts[13] = verts[2];
    pnts[14] = verts[6];
    pnts[15] = verts[7];

    pnts[16] = verts[0];
    pnts[17] = verts[1];
    pnts[18] = verts[2];
    pnts[19] = verts[3];

    pnts[20] = verts[7];
    pnts[21] = verts[6];
    pnts[22] = verts[5];
    pnts[23] = verts[4];

    geom.setCoordinates(0, pnts);

    Appearance app = new Appearance();
    Material mat = new Material(new Color3f(0.0f,0.0f,0.0f), new Color3f(Color.red),new Color3f(0.0f,0.0f,0.0f),new Color3f(0.0f,0.0f,0.0f),110.0f);//ambient, c, dffuse,specular,shininess
    app.setMaterial(mat);
    app.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
    mat.setCapability(Material.ALLOW_COMPONENT_WRITE);
    Alpha a = new Alpha(-1, Alpha.INCREASING_ENABLE,
                        0,0,
                        5000,0,0,
                        0,0,0);
    ColorInterpolator ci = new ColorInterpolator(a, mat, new Color3f(Color.red), new Color3f(Color.blue));
    BoundingSphere bsp = new BoundingSphere(new Point3d(0.0,0.0,0.0), 100.0);
    ci.setSchedulingBounds(bsp);

    Shape3D shape = new Shape3D(geom, app);
    addChild(ci);
    addChild(shape);
 }

}