/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: mirror
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-04-01, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * -------------------------------------------------------------------
  * TODO:
  */
package jaugment2.java3d.objectService;

  //JDK
import java.rmi.RemoteException;
import java.rmi.Remote;
import java.rmi.MarshalledObject;
import java.util.StringTokenizer;
import java.util.LinkedList;
import java.util.Hashtable;
import java.util.Date;
import java.util.Enumeration;
import java.net.URL;
import java.net.InetAddress;
import java.lang.Runnable;
import java.lang.Process;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.LineNumberReader;
  //JINI
import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.UnknownLeaseException;
import net.jini.core.lease.Lease;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;

import net.jini.core.entry.Entry;
import net.jini.core.discovery.LookupLocator;
import net.jini.discovery.DiscoveryListener;
import net.jini.discovery.DiscoveryEvent;
  //project
import jaugment2.jini.simpleService.simpleService;
import jaugment2.jini.SimpleLandlord;
import jaugment2.jini.SimpleLandlord.LeaseListener;
import jaugment2.jini.SimpleLandlord.EventRegistration;
import jaugment2.helpers.Logger;
import jaugment2.UI.java3dUI.Java3DUIFactory;


//helper-mark for configure to determine that rmic needs to be executed for this file
//rmimakehelper


/**
 * Service providing a static object
 */
public class ObjectService extends simpleService implements Remote
{

  //--------------------- initiaslisation&administration ------------------------------------------------------//

  private ObjectServiceProxy myproxy;
  private String lastmsg = null;
  private static final boolean debugThis = false;

  /**
   * Load our state (e.g. attrubutes, service-id) from persistent storage
   */
  private void loadMe()
  {
      String filename = System.getProperty("jaugment2.java3d.mirror.objectService.persistentStateFilename");
      if(filename==null)
        {
         filename = "ObjectServiceState.dat";
         System.setProperty("jaugment2.java3d.objectService.ObjectService.persistentStateFilename", filename);
        };

      persistentState.setProperty("persistentStateFilename", filename);

   load();


   if(persistentState.get("attr")==null)
     {
      Entry[] attrSets = new Entry[]
                   {
                     new net.jini.lookup.entry.Name("objectService"),
                     new net.jini.lookup.entry.Comment("a service providing a static 3D-object"),
                     new net.jini.lookup.entry.ServiceInfo("objectService", "Marcus Wolschon <Suran@gmx.net>", "Marcus Wolschon <Suran@gmx.net>", "0.0.1", "standart", "00000"),

                     ObjectServiceJava3DRepresentation.getUIDescriptor()
                    };
      persistentState.put("attr", attrSets);
     };

   save();
  };


  public ObjectService() throws RemoteException
  {
   this(null);
  };

  public ObjectService(LookupLocator locallocator) throws RemoteException
  {
   if(debugThis)Logger.debug.println("DEBUG: [ObjectService] creating myself...");
   try
     {
      String urls = System.getProperty("jaugment2.java3d.objectService.ObjectService.sceneURL");
      if(urls != null)
        {
         sceneURL = new java.net.URL(urls);
        }
      else
        sceneURL = new java.net.URL("file://World.wrl");

      persistentState.setProperty("groups", "public, Suran");
      persistentState.setProperty("locators", "jini://localhost, jini://127.0.0.1");

      myproxy = new ObjectServiceProxy(this);

      loadMe();


      Entry[] attrSets = (Entry[])persistentState.get("attr");

      this.init(myproxy, attrSets, this, locallocator);
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     };
  };



 /** restart this service once it has been stopped
   * @see stop
   */
 public void restart() throws Exception
 {
 }

  private URL sceneURL;
  public java.net.URL getSceneURL()
  {
   return(sceneURL);
  }

 /** stop this service. If exit is false all references to this
   * service are still valid but it has deregistered everywhere.
   * (this includes the RMI-activation-daemon)
   * @see restart
   * @see destroy
   * @param exit stop running
   */
 public void stop(boolean exit) throws Exception
 {
 }

 /** stop this service, destroy any persistent storage and exit
   */
 public void destroy() throws RemoteException
 {
  deinit();
 }
}