/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Java3D-UI
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-04-08, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  *       - clicking (mouse/hand) works
  * -------------------------------------------------------------------
  * TODO:
  *       - texture
  *       - animation for clicking
  */
package jaugment2.java3d.widgets;

  //JDK
import java.lang.reflect.Array;
import java.awt.AWTEvent;
import java.awt.event.MouseEvent;
  //Java3D
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Behavior;
import javax.media.j3d.Shape3D;
import javax.media.j3d.QuadArray;
import javax.media.j3d.WakeupOnAWTEvent;
import javax.media.j3d.WakeupOnCollisionEntry;
import javax.media.j3d.*;
import com.sun.j3d.utils.picking.PickCanvas;
import com.sun.j3d.utils.picking.PickResult;
import com.sun.j3d.utils.picking.PickTool;
import com.sun.j3d.utils.geometry.Text2D;
import javax.vecmath.Point3f;
import javax.vecmath.*;
  //jAugment
import jaugment2.helpers.Logger;
import jaugment2.UI.java3dUI.runeMagic.AutomataBehavior;
import jaugment2.UI.java3dUI.runeMagic.AutomataBehavior.State;
import jaugment2.UI.java3dUI.runeMagic.AutomataBehavior.Transition;
import jaugment2.UI.java3dUI.runeMagic.AutomataBehavior.InvokeAction;
import jaugment2.UI.java3dUI.runeMagic.AutomataBehavior.ListAction;
import jaugment2.UI.java3dUI.runeMagic.AutomataBehavior.*;

/**
 * A button as a 3-dimensional widget
 */
public class JButton3D extends BranchGroup
{

 private static final boolean debugThis = true;
 private BoundingBox    bounds = new BoundingBox(new Point3d(-0.5f, -0.5f,  -0.5f), new Point3d(0.5f, 0.5f,  0.5f));
 private Shape3D newCube(float minX, float maxX, float minY, float maxY, float minZ, float maxZ, javax.media.j3d.Appearance app)
 {
   Point3f verts[] = new Point3f[8];
   float widthadj = (maxX - minX)*0.1f;
   float depthadj = (maxZ - minZ)*0.1f;
    verts[0] = new Point3f( maxX-widthadj, maxY, maxZ-depthadj);
    verts[1] = new Point3f( minX+widthadj, maxY, maxZ-depthadj);
    verts[2] = new Point3f( minX, minY, maxZ);
    verts[3] = new Point3f( maxX, minY, maxZ);
    verts[4] = new Point3f( maxX-widthadj, maxY, minZ+depthadj);
    verts[5] = new Point3f( minX+widthadj, maxY, minZ+depthadj);
    verts[6] = new Point3f( minX, minY, minZ);
    verts[7] = new Point3f( maxX, minY, minZ);

    QuadArray geom = new QuadArray(24, GeometryArray.COORDINATES/* | GeometryArray.TEXTURE_COORDINATE_3*/);

    Point3f pnts[] = new Point3f[24];
    pnts[0] = verts[0];
    pnts[1] = verts[3];
    pnts[2] = verts[7];
    pnts[3] = verts[4];

    pnts[4] = verts[1];
    pnts[5] = verts[5];
    pnts[6] = verts[6];
    pnts[7] = verts[2];

    pnts[8]  = verts[0];
    pnts[9]  = verts[4];
    pnts[10] = verts[5];
    pnts[11] = verts[1];

    pnts[12] = verts[3];
    pnts[13] = verts[2];
    pnts[14] = verts[6];
    pnts[15] = verts[7];

    pnts[16] = verts[0];
    pnts[17] = verts[1];
    pnts[18] = verts[2];
    pnts[19] = verts[3];

    pnts[20] = verts[7];
    pnts[21] = verts[6];
    pnts[22] = verts[5];
    pnts[23] = verts[4];

    Point2f txtc[] = new Point2f[24];

  /*  for(int i=0; i<24; i+=4)
       {
        txtc[i+3] = new Point2f(1.0f, 0.0f);
        txtc[i+0] = new Point2f(1.0f, 1.0f);
        txtc[i+1] = new Point2f(0.0f, 1.0f);
        txtc[i+2] = new Point2f(0.0f, 0.0f);
       }*/

    geom.setCoordinates(0, pnts);
//    geom.setTextureCoordinates(0, txtc);

    return(new Shape3D(geom, app));
 }

  public JButton3D(float width, float height, float depth, String text, Canvas3D canvas3D)
  {
   super();
   init(width, height, depth, text, canvas3D);
  }

  public JButton3D(float width, float height, float depth, String text)
  {
   super();
   init(width, height, depth, text, null);
  }

  public JButton3D(String text, Canvas3D canvas3D)
  {
   super();
   init(0.2f, 0.05f, 0.1f, text, canvas3D);
  }

  public JButton3D(String text)
  {
   super();
   init(0.2f, 0.05f, 0.1f, text, null);
  }

  private PickCanvas pickCanvas = null;

  private void init(float width, float height, float depth, String text, Canvas3D canvas3D)
  {
   javax.media.j3d.Appearance app = new javax.media.j3d.Appearance();
   if(app.getColoringAttributes() == null)
     app.setColoringAttributes(new ColoringAttributes());
   app.getColoringAttributes().setShadeModel(ColoringAttributes.SHADE_GOURAUD);

     // create button-body
   Shape3D myshape = newCube(0.0f, width, 0.0f, height, 0.0f, depth, app);
   PickTool.setCapabilities(myshape, PickTool.INTERSECT_FULL);
   this.addChild(myshape);

      // create text
   Text2D t2d = new Text2D(text, new Color3f(java.awt.Color.red), "Serif", 20, java.awt.Font.BOLD);
//   t2d.setRectangleScaleFactor(0.25f);
   PickTool.setCapabilities(t2d, PickTool.INTERSECT_FULL);
   Appearance apptxt = t2d.getAppearance();
   if(apptxt != null)
   if(apptxt.getPolygonAttributes() == null)
     {
      PolygonAttributes papp = new PolygonAttributes();
      papp.setCullFace(PolygonAttributes.CULL_NONE);
      apptxt.setPolygonAttributes(papp);
     }
    Transform3D txttrans = new Transform3D();
    txttrans.rotX(java.lang.Math.toRadians(90.0));
    txttrans.setTranslation(new Vector3f(width*0.1f, height*1.01f, depth*(-0.1f)));
    TransformGroup txttg = new TransformGroup(txttrans);
    txttg.addChild(t2d);
    this.addChild(txttg);

      // initialize picking
   if(canvas3D != null)
     {
      pickCanvas = new PickCanvas(canvas3D, this);
      pickCanvas.setTolerance(5.0f);
      pickCanvas.setMode(PickCanvas.GEOMETRY);
     }


     // create behavior
   String what = "";
   try
     {
      what = "creating State";
      State s = new State(new Transition[0]);

      what = "creating pressed-action";
       Action actions_pressed[] = new Action[] {
            new StimulusPushValueAction(new Object[0], Class.forName("javax.media.j3d.WakeupOnAWTEvent").getMethod("getAWTEvent", new Class[0]))
          , new PopInvokeAction(this, 1, this.getClass().getMethod("processAWTEvents", new Class[] {Array.newInstance(Class.forName("java.awt.AWTEvent"),0).getClass()}))
//          new InvokeAction(this, this.getClass().getMethod("press", new Class[0]))
         };
      Transition mouseTrans1 = new Transition(s,
                                              new WakeupOnAWTEvent( java.awt.event.MouseEvent.MOUSE_PRESSED ),
                                              new ListAction(actions_pressed)
                                             );
      s.addTransition(mouseTrans1);


      what = "creating WakeupOnCollisionEntry";
      Action actions_collision[] = new Action[] {
          new InvokeAction(this, this.getClass().getMethod("press", new Class[0]))
         };
      Transition collisionTrans = new Transition(s,
                                                 new WakeupOnCollisionEntry(this, WakeupOnCollisionEntry.USE_GEOMETRY ),
                                                 new ListAction(actions_collision));
      s.addTransition(collisionTrans);


      what = "creating automata";
      AutomataBehavior behavior = new AutomataBehavior(new Automata(new State[]{s}, s));
      behavior.setSchedulingBounds(bounds);
      addChild(behavior);
     }
   catch(Exception e)
     {
      Logger.err.println("ERROR: of type ["+e.getClass().toString()+"] while "+what);
      e.printStackTrace(Logger.err);
     }
  }

 /**
  * Check if our geometry was picked
  */
 private boolean check(int x, int y)
 {
//  Logger.debug.println("DEBUG: [ExamineViewBehavior] check(x="+x+", y="+y+")");
  pickCanvas.setShapeLocation(x, y);
 // PickResult pr[] = pickCanvas.pickAllSorted();
   PickResult pr = pickCanvas.pickClosest();
  if(debugThis)Logger.debug.println("DEBUG: [JButton3D] check(x="+x+", y="+y+") returning "+(pr != null));
  return(pr != null);
//  return(true);
 }

 public void processAWTEvents( AWTEvent me2[] )
 {
  for(int i=0; i<me2.length;i++)
  if(me2[i] instanceof MouseEvent)
    {
     if(me2[i].getSource() instanceof Canvas3D)
     if((pickCanvas == null) || ( me2[i].getSource() != pickCanvas.getCanvas()))
       {
        pickCanvas = new PickCanvas((Canvas3D)me2[i].getSource(), this);
        pickCanvas.setTolerance(5.0f);
        pickCanvas.setMode(PickCanvas.GEOMETRY);
       }

     if(pickCanvas != null)
     if(check( ((MouseEvent)me2[i]).getX() , ((MouseEvent)me2[i]).getY() ))
       press();
    }
 }


  public void press()
  {
   if(debugThis)Logger.debug.println("DEBUG: [JButton3D] press()");
  }
}